@@ -177,6 +177,7 @@ void EvolutionCalendarSource::open()
     string id = getDatabaseID();    
     ESource *source = findSource(sources, id);
     bool onlyIfExists = true;
+    bool created = false;
     if (!source) {
         // might have been special "<<system>>" or "<<default>>", try that and
         // creating address book from file:// URI before giving up
@@ -187,6 +188,7 @@ void EvolutionCalendarSource::open()
         } else {
             throwError(string("not found: '") + id + "'");
         }
+        created = true;
         onlyIfExists = false;
     } else {
         m_calendar.set(e_cal_new(source, m_type), m_typeName.c_str());
@@ -195,10 +197,14 @@ void EvolutionCalendarSource::open()
     e_cal_set_auth_func(m_calendar, eCalAuthFunc, this);
     
     if (!e_cal_open(m_calendar, onlyIfExists, &gerror)) {
-        // opening newly created address books often failed, perhaps that also applies to calendars - try again
-        g_clear_error(&gerror);
-        sleep(5);
-        if (!e_cal_open(m_calendar, onlyIfExists, &gerror)) {
+        if (created) {
+            // opening newly created address books often failed, perhaps that also applies to calendars - try again
+            g_clear_error(&gerror);
+            sleep(5);
+            if (!e_cal_open(m_calendar, onlyIfExists, &gerror)) {
+                throwError(string("opening ") + m_typeName, gerror );
+            }
+        } else {
             throwError(string("opening ") + m_typeName, gerror );
         }
     }
@@ -209,6 +215,15 @@ void EvolutionCalendarSource::open()
                            (void *)"Evolution Data Server has died unexpectedly, database no longer available.");
 }
 
+bool EvolutionCalendarSource::isEmpty()
+{
+    // TODO: add more efficient implementation which does not
+    // depend on actually pulling all items from EDS
+    RevisionMap_t revisions;
+    listAllItems(revisions);
+    return revisions.empty();
+}
+
 void EvolutionCalendarSource::listAllItems(RevisionMap_t &revisions)
 {
     GError *gerror = NULL;
@@ -414,9 +429,30 @@ EvolutionCalendarSource::InsertItemResult EvolutionCalendarSource::insertItem(co
         ItemID id(newluid);
         bool isParent = id.m_rid.empty();
 
-        // ensure that the component has the right UID
-        if (update && !id.m_uid.empty()) {
-            icalcomponent_set_uid(subcomp, id.m_uid.c_str());
+        // ensure that the component has the right UID and
+        // RECURRENCE-ID
+        if (update) {
+            if (!id.m_uid.empty()) {
+                icalcomponent_set_uid(subcomp, id.m_uid.c_str());
+            }
+            if (!id.m_rid.empty()) {
+                // Reconstructing the RECURRENCE-ID is non-trivial,
+                // because our luid only contains the date-time, but
+                // not the time zone. Only do the work if the event
+                // really doesn't have a RECURRENCE-ID.
+                struct icaltimetype rid;
+                rid = icalcomponent_get_recurrenceid(subcomp);
+                if (icaltime_is_null_time(rid)) {
+                    // Preserve the original RECURRENCE-ID, including
+                    // timezone, no matter what the update contains
+                    // (might have wrong timezone or UTC).
+                    eptr<icalcomponent> orig(retrieveItem(id));
+                    icalproperty *orig_rid = icalcomponent_get_first_property(orig, ICAL_RECURRENCEID_PROPERTY);
+                    if (orig_rid) {
+                        icalcomponent_add_property(subcomp, icalproperty_new_clone(orig_rid));
+                    }
+                }
+            }
         }
 
         if (isParent) {
@@ -555,6 +591,27 @@ void EvolutionCalendarSource::removeItem(const string &luid)
         }
     }
     m_allLUIDs.erase(luid);
+
+    if (!id.m_rid.empty()) {
+        // Removing the child may have modified the parent.
+        // We must record the new LAST-MODIFIED string,
+        // otherwise it might be reported as modified during
+        // the next sync (timing dependent: if the parent
+        // was updated before removing the child *and* the
+        // update and remove fall into the same second,
+        // then the modTime does not change again during the
+        // removal).
+        try {
+            ItemID parent(id.m_uid, "");
+            string modTime = getItemModTime(parent);
+            string parentLUID = parent.getLUID();
+            updateRevision(getTrackingNode(), parentLUID, parentLUID, modTime);
+        } catch (...) {
+            // There's no guarantee that the parent still exists.
+            // Instead of checking that, ignore errors (a bit hacky,
+            // but better than breaking the removal).
+        }
+    }
 }
 
 icalcomponent *EvolutionCalendarSource::retrieveItem(const ItemID &id)
