@@ -35,7 +35,7 @@
 # include <execinfo.h>
 #endif
 
-#include <syncevo/SyncContext.h>
+#include "CmdlineSyncClient.h"
 #include "EvolutionSyncSource.h"
 #include <syncevo/util.h>
 #include <syncevo/VolatileConfigNode.h>
@@ -185,36 +185,53 @@ public:
 
         BOOST_FOREACH(const RegisterSyncSourceTest *test, m_configs) {
             if (sources.find(test->m_configName) != sources.end()) {
-                m_source2Config.push_back(test->m_configName);
+                m_syncSource2Config.push_back(test->m_configName);
             }
         }
 
+        /* Local Test SyncSource : remove all virtual datastores, inserting the
+         * sub datastores*/
+        ClientTest::Config conf;
+        BOOST_FOREACH (string source, sources) {
+            getSourceConfig (source, conf);
+            if (conf.subConfigs) {
+                vector<string> subs;
+                boost::split (subs, conf.subConfigs, boost::is_any_of(","));
+                BOOST_FOREACH (string sub, subs) {
+                    m_localSource2Config.push_back (sub);
+                }
+            } else {
+                m_localSource2Config.push_back (source);
+            }
+        }
         // get configuration and set obligatory fields
         LoggerBase::instance().setLevel(Logger::DEBUG);
         std::string root = std::string("evolution/") + server + "_" + id;
-        SyncConfig config(string(server) + "_" + id);
+        boost::shared_ptr<SyncConfig> config(new SyncConfig(string(server) + "_" + id));
         boost::shared_ptr<SyncConfig> from = boost::shared_ptr<SyncConfig> ();
 
-        if (!config.exists()) {
-            // no configuration yet
-            config.setDefaults();
-            from = SyncConfig::createServerTemplate(server);
+        if (!config->exists()) {
+            // no configuration yet, create in different contexts because
+            // device ID is different
+            config.reset(new SyncConfig(string(server) + "_" + id + "@client-test-" + id));
+            config->setDefaults();
+            from = SyncConfig::createPeerTemplate(server);
             if(from) {
                 set<string> filter;
-                config.copy(*from, &filter);
+                config->copy(*from, &filter);
             }
-            config.setDevID(id == "1" ? "sc-api-nat" : "sc-pim-ppc");
+            config->setDevID(id == "1" ? "sc-api-nat" : "sc-pim-ppc");
         }
         BOOST_FOREACH(const RegisterSyncSourceTest *test, m_configs) {
             ClientTest::Config testconfig;
             getSourceConfig(test, testconfig);
             CPPUNIT_ASSERT(testconfig.type);
 
-            boost::shared_ptr<SyncSourceConfig> sc = config.getSyncSourceConfig(testconfig.sourceName);
+            boost::shared_ptr<SyncSourceConfig> sc = config->getSyncSourceConfig(testconfig.sourceName);
             if (!sc || !sc->exists()) {
                 // no configuration yet
-                config.setSourceDefaults(testconfig.sourceName);
-                sc = config.getSyncSourceConfig(testconfig.sourceName);
+                config->setSourceDefaults(testconfig.sourceName);
+                sc = config->getSyncSourceConfig(testconfig.sourceName);
                 CPPUNIT_ASSERT(sc);
                 sc->setURI(testconfig.uri);
                 if(from && testconfig.sourceNameServerTemplate){
@@ -230,19 +247,41 @@ public:
             sc->setUser(m_evoUser);
             sc->setPassword(m_evoPassword);
         }
-        config.flush();
+        config->flush();
     }
 
     virtual LocalTests *createLocalTests(const std::string &name, int sourceParam, ClientTest::Config &co) {
         return new EvolutionLocalTests(name, *this, sourceParam, co);
     }
 
-    virtual int getNumSources() {
-        return m_source2Config.size();
+    virtual int getNumLocalSources() {
+        return m_localSource2Config.size();
     }
 
-    virtual void getSourceConfig(int source, Config &config) {
-        getSourceConfig(m_configs[m_source2Config[source]], config);
+    virtual int getNumSyncSources() {
+        return m_syncSource2Config.size();
+    }
+
+    virtual void getLocalSourceConfig(int source, Config &config) {
+        getSourceConfig(m_configs[m_localSource2Config[source]], config);
+    }
+
+    virtual void getSyncSourceConfig(int source, Config &config) {
+        getSourceConfig(m_configs[m_syncSource2Config[source]], config);
+    }
+
+    virtual int getLocalSourcePosition(const string &configName) {
+        for (size_t i=0; i< m_localSource2Config.size(); i++) {
+            if(m_localSource2Config[i] == configName) {
+                return i;
+                break;
+            }
+        }
+        return -1;
+    }
+
+    virtual void getSourceConfig (const string &configName, Config &config) {
+        return getSourceConfig (m_configs[configName], config);
     }
 
     static void getSourceConfig(const RegisterSyncSourceTest *test, Config &config) {
@@ -267,22 +306,16 @@ public:
                                 const std::string &logbase,
                                 const SyncOptions &options)
     {
-        set<string> activeSources;
-        for(int i = 0; sources[i] >= 0; i++) {
-            activeSources.insert(m_source2Config[sources[i]]);
-        }
-
         string server = getenv("CLIENT_TEST_SERVER") ? getenv("CLIENT_TEST_SERVER") : "funambol";
         server += "_";
         server += m_clientID;
         
-        class ClientTest : public SyncContext {
+        class ClientTest : public CmdlineSyncClient {
         public:
             ClientTest(const string &server,
-                       const set<string> &activeSources,
                        const string &logbase,
                        const SyncOptions &options) :
-                SyncContext(server, false, activeSources),
+                CmdlineSyncClient(server, false, true),
                 m_logbase(logbase),
                 m_options(options),
                 m_started(false)
@@ -295,13 +328,10 @@ public:
                 setMaxObjSize(m_options.m_maxObjSize, true);
                 setMaxMsgSize(m_options.m_maxMsgSize, true);
                 setWBXML(m_options.m_isWBXML, true);
+                setRetryDuration(m_options.m_retryDuration, true);
+                setRetryInterval(m_options.m_retryInterval, true);
                 SyncContext::prepare();
             }
-            virtual void prepare(const std::vector<SyncSource *> &sources) {
-                SyncModes modes(m_options.m_syncMode);
-                setSyncModes(sources, modes);
-                SyncContext::prepare(sources);
-            }
 
             virtual void displaySyncProgress(sysync::TProgressEventEnum type,
                                              int32_t extra1, int32_t extra2, int32_t extra3)
@@ -332,7 +362,18 @@ public:
             const string m_logbase;
             SyncOptions m_options;
             bool m_started;
-        } client(server, activeSources, logbase, options);
+        } client(server, logbase, options);
+
+        // configure active sources with the desired sync mode,
+        // disable the rest
+        FilterConfigNode::ConfigFilter filter;
+        filter[SyncSourceConfig::m_sourcePropSync.getName()] = "none";
+        client.setConfigFilter(false, "", filter);
+        filter[SyncSourceConfig::m_sourcePropSync.getName()] =
+            PrettyPrintSyncMode(options.m_syncMode);
+        for(int i = 0; sources[i] >= 0; i++) {
+            client.setConfigFilter(false, m_syncSource2Config[sources[i]], filter);
+        }
 
         SyncReport report;
         SyncMLStatus status = client.sync(&report);
@@ -353,18 +394,22 @@ private:
      * This is the mapping to the corresponding config name, created when
      * constructing this instance.
      */
-    vector<string> m_source2Config;
+    vector<string> m_localSource2Config;
+    vector<string> m_syncSource2Config;
 
     /** returns the name of the Evolution database */
     string getDatabaseName(const string &configName) {
+        if (configName == "calendar+todo") {
+            return "ical20,itodo20";
+        } else if (configName == "file_calendar+todo") {
+            return "file_ical20,file_itodo20";
+        }
         return m_evoPrefix + configName + "_" + m_clientID;
     }
     
     static TestingSyncSource *createSource(ClientTest &client, int source, bool isSourceA) {
         TestEvolution &evClient((TestEvolution &)client);
-        string changeID = "SyncEvolution Change ID #";
-        string name = evClient.m_source2Config[source];
-        changeID += isSourceA ? "1" : "2";
+        string name = evClient.m_localSource2Config[source];
         string database = evClient.getDatabaseName(name);
 
         SyncConfig config("client-test-changes");
@@ -373,13 +418,12 @@ private:
                                                           "_" + (isSourceA ? "A" : "B"));
 
         // always set this property: the name might have changes since last test run
-        nodes.m_configNode->setProperty("evolutionsource", database.c_str());
-        nodes.m_configNode->setProperty("evolutionuser", evClient.m_evoUser.c_str());
-        nodes.m_configNode->setProperty("evolutionpassword", evClient.m_evoPassword.c_str());
+        nodes.getProperties()->setProperty("evolutionsource", database.c_str());
+        nodes.getProperties()->setProperty("evolutionuser", evClient.m_evoUser.c_str());
+        nodes.getProperties()->setProperty("evolutionpassword", evClient.m_evoPassword.c_str());
 
         SyncSourceParams params(name,
-                                nodes,
-                                changeID);
+                                nodes);
 
         const RegisterSyncSourceTest *test = evClient.m_configs[name];
         ClientTestConfig testConfig;
@@ -433,6 +477,7 @@ public:
         // this is required when using glib directly or indirectly
         g_type_init();
         g_thread_init(NULL);
+        g_set_prgname("client-test");
 #endif
         EDSAbiWrapperInit();
         testClient.registerTests();
