@@ -43,6 +43,7 @@ class SyncSource;
 
 struct SuspendFlags
 {
+    /** SIGINT twice within this amount of seconds aborts the sync */
     static const time_t ABORT_INTERVAL = 2; 
     enum CLIENT_STATE
     {
@@ -53,7 +54,16 @@ struct SuspendFlags
     }state;
     time_t last_suspend;
 
-    SuspendFlags():state(CLIENT_NORMAL),last_suspend(0)
+    /**
+     * Simple string to print in SyncContext::printSignals().
+     * Set by SyncContext::handleSignal() when updating the
+     * global state. There's a slight race condition: if
+     * messages are set more quickly than they are printed,
+     * only the last message is printed.
+     */
+    const char *message;
+
+SuspendFlags():state(CLIENT_NORMAL),last_suspend(0),message(NULL)
     {}
 };
 
@@ -68,22 +78,52 @@ struct SuspendFlags
  *
  */
 class SyncContext : public SyncConfig, public ConfigUserInterface {
+    /**
+     * the string used to request a config,
+     * *not* the normalized config name itself;
+     * for that use SyncConfig::getConfigName()
+     */
     const string m_server;
-    const set<string> m_sources;
-    const bool m_doLogging;
+
+    bool m_doLogging;
     bool m_quiet;
     bool m_dryrun;
 
+    bool m_serverMode;
+    std::string m_sessionID;
+    SharedBuffer m_initialMessage;
+    string m_initialMessageType;
+    string m_syncDeviceID;
+
+    
+    boost::shared_ptr<TransportAgent> m_agent;
     /**
      * flags for suspend and abort
      */
     static SuspendFlags s_flags;
 
     /**
-     * a pointer to the active SourceList instance if one exists; 
+     * a pointer to the active SourceList instance for this context if one exists;
      * used for error handling in throwError() on the iPhone
      */
-    static SourceList *m_sourceListPtr;
+    SourceList *m_sourceListPtr;
+
+    /**
+     * a pointer to the active SyncContext instance if one exists;
+     * set by sync() and/or SwapContext
+     */
+    static SyncContext *m_activeContext;
+    class SwapContext {
+        SyncContext *m_oldContext;
+    public:
+        SwapContext(SyncContext *newContext) :
+            m_oldContext(SyncContext::m_activeContext) {
+            SyncContext::m_activeContext = newContext;
+        }
+        ~SwapContext() {
+            SyncContext::m_activeContext = m_oldContext;
+        }
+    };
 
     /**
      * Connection to the Synthesis engine. Always valid in a
@@ -113,17 +153,37 @@ class SyncContext : public SyncConfig, public ConfigUserInterface {
         }
     };
 
+    /*
+     * The URL this SyncContext is actually using, since we may support multiple
+     * urls in the configuration.
+     * */
+    string m_usedSyncURL;
+
+    /* Indicates whether current sync session is triggered by remote peer
+     * (such as server alerted sync)
+     */
+    bool m_remoteInitiated;
   public:
+    /**
+     * SyncContext using a volatile config
+     * and no logging.
+     */
+    SyncContext();
+
     /**
      * @param server     identifies the server config to be used
      * @param doLogging  write additional log and datatbase files about the sync
      */
     SyncContext(const string &server,
-                        bool doLogging = false,
-                        const set<string> &sources = set<string>());
+                bool doLogging = false);
     ~SyncContext();
 
-    using SyncConfig::savePassword;
+    /**
+     * Output channel to be used by this context. NULL means "use std::cout", the default.
+     * Owned by caller, must remain valid as long as SyncContext exists.
+     */
+    void setOutput(ostream *out);
+    ostream &getOutput() const { return *m_out; }
 
     bool getQuiet() { return m_quiet; }
     void setQuiet(bool quiet) { m_quiet = quiet; }
@@ -131,7 +191,36 @@ class SyncContext : public SyncConfig, public ConfigUserInterface {
     bool getDryRun() { return m_dryrun; }
     void setDryRun(bool dryrun) { m_dryrun = dryrun; }
 
-    static SuspendFlags& getSuspendFlags() {return s_flags;}
+    /** only for server: device ID of peer */
+    void setSyncDeviceID(const std::string &deviceID) { m_syncDeviceID = deviceID; }
+    std::string getSyncDeviceID() const { return m_syncDeviceID; }
+
+    /** read-only access to suspend and abort state */
+    static const SuspendFlags &getSuspendFlags() { return s_flags; }
+
+    /*
+     * Use sendSAN as the first step is sync() if this is a server alerted sync.
+     * Prepare the san package and send the SAN request to the peer.
+     * Returns false if failed to get a valid client sync request
+     * otherwise put the client sync request into m_initialMessage which will
+     * be used to initalze the server via initServer(), then continue sync() to
+     * start the real sync serssion.
+     * @version indicates the SAN protocal version used (1.2 or 1.1/1.0)
+     */
+    bool sendSAN(uint16_t version);
+
+    /**
+     * Initializes the session so that it runs as SyncML server once
+     * sync() is called. For this to work the first client message
+     * must be available already.
+     *
+     * @param sessionID    session ID to be used by server
+     * @param data         content of initial message sent by the client
+     * @param messageType  content type set by the client
+     */
+    void initServer(const std::string &sessionID,
+                    SharedBuffer data,
+                    const std::string &messageType);
 
     /**
      * Executes the sync, throws an exception in case of failure.
@@ -142,6 +231,35 @@ class SyncContext : public SyncConfig, public ConfigUserInterface {
      */
     SyncMLStatus sync(SyncReport *report = NULL);
 
+    /** result of analyzeSyncMLMessage() */
+    struct SyncMLMessageInfo {
+        std::string m_deviceID;
+
+        /** a string representation of the whole structure for debugging */
+        std::string toString() { return std::string("deviceID ") + m_deviceID; }
+    };
+
+    /**
+     * Instead or executing a sync, analyze the initial message
+     * without changing any local data. Returns once the LocURI =
+     * device ID of the client is known.
+     *
+     * @return device ID, empty if not in data
+     */
+    static SyncMLMessageInfo
+        analyzeSyncMLMessage(const char *data, size_t len,
+                             const std::string &messageType);
+
+    /**
+     * Convenience function, to be called inside a catch() block of
+     * (or for) the sync.
+     *
+     * Rethrows the exception to determine what it is, then logs it
+     * as an error and returns a suitable error code (usually a general
+     * STATUS_DATASTORE_FAILURE).
+     */
+    SyncMLStatus handleException();
+
     /**
      * Determines the log directory of the previous sync (either in
      * temp or logdir) and shows changes since then.
@@ -174,8 +292,9 @@ class SyncContext : public SyncConfig, public ConfigUserInterface {
 
     /**
      * fills report with information about previous session
+     * @return the peer name from the dir.
      */
-    void readSessionInfo(const string &dir, SyncReport &report);
+    string readSessionInfo(const string &dir, SyncReport &report);
 
     /**
      * fills report with information about local changes
@@ -243,32 +362,76 @@ class SyncContext : public SyncConfig, public ConfigUserInterface {
      * session. Called by Synthesis DB plugin to find active
      * sources.
      *
+     * @param name     can be both <SyncSource::getName()> as well as <prefix>_<SyncSource::getName()>
+     *                 (necessary when renaming sources in the Synthesis XML config)
+     *
      * @TODO: roll SourceList into SyncContext and
      * make this non-static
      */
     static SyncSource *findSource(const char *name);
+    static const char m_findSourceSeparator = '@';
 
     /**
-     * intercept config filters
+     * Find the active sync context for the given session.
      *
-     * This call removes the "sync" source property and remembers
-     * it separately because it has to be applied to only the active
-     * sync sources; the generic config handling code would apply
-     * it to all sources.
+     * @param sessionName      chosen by SyncEvolution and passed to
+     *                         Synthesis engine, which calls us back
+     *                         with it in SyncEvolution_Session_CreateContext()
+     * @return context or NULL if not found
      */
-    virtual void setConfigFilter(bool sync, const FilterConfigNode::ConfigFilter &filter);
+    static SyncContext *findContext(const char *sessionName);
 
     SharedEngine getEngine() { return m_engine; }
     const SharedEngine getEngine() const { return m_engine; }
 
     bool getDoLogging() { return m_doLogging; }
-    std::string getServer() { return m_server; }
+
+    /**
+     * Returns the string used to select the peer config
+     * used by this instance.
+     *
+     * Note that this is not the same as a valid configuration
+     * name. For example "foo" might be matched against a
+     * "foo@bar" config by SyncConfig. Use SyncConfig::getConfigName()
+     * to get the underlying config.
+     */
+    std::string getPeer() { return m_server; }
 
     /**
      * Handle for active session, may be NULL.
      */
     SharedSession getSession() { return m_session; }
 
+    /**
+     * sync() installs signal handlers for SIGINT and SIGTERM if no
+     * handler was installed already. SIGINT will try to suspend.
+     * Sending the signal again quickly (typically done by pressing
+     * CTRL-C twice) will abort. SIGTERM will abort the running sync
+     * immediately.
+     *
+     * If a handler was installed already, the caller is responsible
+     * for calling this function if this kind of SIGINT/SIGTERM
+     * handling is desired.
+     */
+    static void handleSignal(int signal);
+
+    /**
+     * Once a signal was received, all future calls to sync() will
+     * react to it unless this function is called first.
+     */
+    static void resetSignals() { s_flags = SuspendFlags(); }
+
+    /**
+     * handleSignals() is called in a signal handler,
+     * which can only call reentrant functions. Our
+     * logging code is not reentrant and thus has
+     * to be called outside of the signal handler.
+     */
+    static void printSignals();
+
+    bool getRemoteInitiated() {return m_remoteInitiated;}
+    void setRemoteInitiated(bool remote) {m_remoteInitiated = remote;}
+
   protected:
     /** exchange active Synthesis engine */
     SharedEngine swapEngine(SharedEngine newengine) {
@@ -345,20 +508,38 @@ class SyncContext : public SyncConfig, public ConfigUserInterface {
     /**
      * Return skeleton Synthesis client XML configuration.
      *
-     * If it contains a <datastore/> element, then that element will
-     * be replaced by the configurations of all active sync
-     * sources. Otherwise the configuration is used as-is.
+     * The <scripting/>, <datatypes/>, <clientorserver/> elements (if
+     * present) are replaced by the caller with fragments found in the
+     * file system. When <datatypes> already has content, that content
+     * may contain <fieldlists/>, <profiles/>, <datatypedefs/>, which
+     * will be replaced by definitions gathered from backends.
      *
      * The default implementation of this function takes the configuration from
      * (in this order):
-     * - ./syncevolution.xml
-     * - <server config dir>/syncevolution.xml
-     * - built-in default
+     * - $(XDG_CONFIG_HOME)/syncevolution-xml
+     * - $(datadir)/syncevolution/xml
+     * Files with identical names are read from the first location where they
+     * are found. If $(SYNCEVOLUTION_XML_CONFIG_DIR) is set, then it overrides
+     * the previous two locations.
      *
+     * The syncevolution.xml file is read from the first place where it is found.
+     * In addition, further .xml files in sub-directories are gathered and get
+     * inserted into the syncevolution.xml template.
+     *
+     * If none of these locations has XML configs, then builtin strings are
+     * used as fallback. This only works for mode == "client". Otherwise an
+     * error is thrown.
+     *
+     * @param mode         "client" or "server"
      * @retval xml         is filled with Synthesis client config which may hav <datastore/>
+     * @retval rules       remote rules which the caller needs for <clientorserver/>
      * @retval configname  a string describing where the config came from
      */
-    virtual void getConfigTemplateXML(string &xml, string &configname);
+    virtual void getConfigTemplateXML(const string &mode,
+                                      string &xml,
+                                      string &rules,
+                                      string &configname);
+                                      
 
     /**
      * Return complete Synthesis XML configuration.
@@ -404,19 +585,10 @@ class SyncContext : public SyncConfig, public ConfigUserInterface {
      */
     virtual void prepare() {}
 
-    /**
-     * Callback for derived classes: called after setting up the client's
-     * and sources' configuration. Can be used to reconfigure sources before
-     * actually starting the synchronization.
-     *
-     * @param sources   a NULL terminated array of all active sources
-     */
-    virtual void prepare(const std::vector<SyncSource *> &sources) {}
-
     /**
      * instantiate transport agent
      *
-     * Called by engine when it needs to do HTTP POST requests.  The
+     * Called by engine when it needs to exchange messages.  The
      * transport agent will be used throughout the sync session and
      * unref'ed when no longer needed. At most one agent will be
      * requested at a time. The transport agent is intentionally
@@ -425,12 +597,22 @@ class SyncContext : public SyncConfig, public ConfigUserInterface {
      * keeping a reference or by returning a shared_ptr where the
      * destructor doesn't do anything.
      *
+     * The agent must be ready for use:
+     * - HTTP specific settings must have been applied
+     * - the current SyncContect's transport_cb() must have been
+     *   installed via TransportAgent::setCallback(), with a suitable
+     *   timeout for the agent
+     *
      * The default implementation instantiates one of the builtin
      * transport agents, depending on how it was compiled.
      *
+     * @param gmainloop    the GMainLoop to be used by transports, if not NULL;
+     *                     transports not supporting that should not be created;
+     *                     transports will increase the reference count for the loop
      * @return transport agent
      */
-    virtual boost::shared_ptr<TransportAgent> createTransportAgent();
+    virtual boost::shared_ptr<TransportAgent> createTransportAgent(void *gmainloop);
+    virtual boost::shared_ptr<TransportAgent> createTransportAgent() { return createTransportAgent(NULL); }
 
     /**
      * display a text message from the server
@@ -466,6 +648,15 @@ class SyncContext : public SyncConfig, public ConfigUserInterface {
                                        SyncSource &source,
                                        int32_t extra1, int32_t extra2, int32_t extra3);
 
+    /**
+     * report step command info
+     *
+     * Will be called after each step in step loop in SyncContext::doSync().
+     * This reports step command info. 
+     * @param stepCmd step command enum value 
+     */
+    virtual void reportStepCmd(sysync::uInt16 stepCmd) {}
+
     /**
      * Called to find out whether user wants to abort sync.
      *
@@ -477,7 +668,10 @@ class SyncContext : public SyncConfig, public ConfigUserInterface {
      *
      * @return true if user wants to abort
      */
-    virtual bool checkForAbort() { return (s_flags.state == SuspendFlags::CLIENT_ABORT);}
+    virtual bool checkForAbort() {
+        printSignals();
+        return (s_flags.state == SuspendFlags::CLIENT_ABORT);
+    }
 
     /**
      * Called to find out whether user wants to suspend sync.
@@ -485,37 +679,61 @@ class SyncContext : public SyncConfig, public ConfigUserInterface {
      * Same as checkForAbort(), but the session is finished
      * gracefully so that it can be resumed.
      */
-    virtual bool checkForSuspend() { return (s_flags.state == SuspendFlags::CLIENT_SUSPEND);}
+    virtual bool checkForSuspend() {
+        printSignals();
+        return (s_flags.state == SuspendFlags::CLIENT_SUSPEND);
+    }
 
  private:
+    /** initialize members as part of constructors */
+    void init();
+
+    /**
+     * generate XML configuration and (re)initialize engine with it
+     */
+    void initEngine(bool logXML);
+
     /**
      * the code common to init() and status():
      * populate source list with active sources and open
      */
     void initSources(SourceList &sourceList);
 
+    /**
+     * called by SynthesDBPlugin in SyncEvolution_StartDataRead()
+     */
+    void startSourceAccess(SyncSource *source);
+
     /**
      * utility function for status() and getChanges():
      * iterate over sources, check for changes and copy result
      */
     void checkSourceChanges(SourceList &sourceList, SyncReport &changes);
 
+    /**
+     * A method to report sync is really successfully started.
+     * It happens at the same time SynthesDBPlugin starts to access source.
+     * For each sync, it is only called at most one time.
+     * The default action is nothing.
+     */
+    virtual void syncSuccessStart() { }
+
     /**
      * sets up Synthesis session and executes it
      */
     SyncMLStatus doSync();
 
     /**
-     * iterate over files mentioned in getSSLServerCertificates()
-     * and return name of first one which is found, empty string
-     * if none
+     * directory for Synthesis client binfiles or
+     * Synthesis server textdb files, unique for each
+     * peer
      */
-    std::string findSSLServerCertificate();
+    string getSynthesisDatadir() { return getRootPath() + "/.synthesis"; }
 
     /**
-     * override sync mode of all active sync sources if set
+     * return true if "delayedabort" session variable is true
      */
-    string m_overrideMode;
+    bool checkForScriptAbort(SharedSession session);
 
     // total retry duration
     int m_retryDuration;
@@ -524,9 +742,17 @@ class SyncContext : public SyncConfig, public ConfigUserInterface {
     // Current retry count
     int m_retries;
 
+    //a flag indicating whether it is the first time to start source access.
+    //It can be used to report infomation about a sync is successfully started.
+    bool m_firstSourceAccess;
+
+    // output stream to be used by this context, never NULL (uses cout as fallback)
+    ostream *m_out;
+
 public:
     static bool transport_cb (void *data);
-    bool processTransportCb();
+
+    string getUsedSyncURL();
 };
 
 SE_END_CXX
