@@ -26,6 +26,7 @@
 #include <syncevo/SyncSource.h>
 #include <syncevo/SyncContext.h>
 #include <syncevo/util.h>
+#include <syncevo/SafeConfigNode.h>
 
 #include <syncevo/SynthesisEngine.h>
 #include <synthesis/SDK_util.h>
@@ -33,9 +34,13 @@
 
 #include <boost/bind.hpp>
 #include <boost/algorithm/string/join.hpp>
+#include <boost/algorithm/string/split.hpp>
+#include <boost/algorithm/string/classification.hpp>
+#include <boost/algorithm/string/predicate.hpp>
 
 #include <ctype.h>
 #include <errno.h>
+#include <unistd.h>
 
 #include <fstream>
 #include <iostream>
@@ -88,7 +93,10 @@ void SyncSourceBase::getDatastoreXML(string &xml, XMLConfigFragments &fragments)
 
     xmlstream <<
         "      <plugin_module>SyncEvolution</plugin_module>\n"
-        "      <plugin_datastoreadmin>no</plugin_datastoreadmin>\n"
+        "      <plugin_datastoreadmin>" <<
+        (serverModeEnabled() ? "yes" : "no") <<
+        "</plugin_datastoreadmin>\n"
+        "      <fromremoteonlysupport> yes </fromremoteonlysupport>\n"
         "\n"
         "      <!-- General datastore settings for all DB types -->\n"
         "\n"
@@ -136,10 +144,10 @@ void SyncSourceBase::getDatastoreXML(string &xml, XMLConfigFragments &fragments)
                 "           " << info.m_beforeWriteScript << "\n";
         }
         xmlstream <<
-            "           itemdata = MAKETEXTWITHPROFILE(" << info.m_profile << ", \"EVOLUTION\");\n"
+            "           itemdata = MAKETEXTWITHPROFILE(" << info.m_profile << ", \"" << info.m_backendRule << "\");\n"
             "        ]]></beforewritescript>\n"
             "        <afterreadscript><![CDATA[\n"
-            "           PARSETEXTWITHPROFILE(itemdata, " << info.m_profile << ", \"EVOLUTION\");\n";
+            "           PARSETEXTWITHPROFILE(itemdata, " << info.m_profile << ", \"" << info.m_backendRule << "\");\n";
         if(!info.m_afterReadScript.empty()) {
             xmlstream << 
                 "           " << info.m_afterReadScript<< "\n";
@@ -301,11 +309,20 @@ string SyncSource::backendsDebug() {
     return scannedModules.debug.str();
 }
 
-SyncSource *SyncSource::createSource(const SyncSourceParams &params, bool error)
+SyncSource *SyncSource::createSource(const SyncSourceParams &params, bool error, SyncConfig *config)
 {
     string sourceTypeString = getSourceTypeString(params.m_nodes);
     SourceType sourceType = getSourceType(params.m_nodes);
 
+    if (sourceType.m_backend == "virtual") {
+        SyncSource *source = NULL;
+        source = new VirtualSyncSource(params, config);
+        if (error && !source) {
+            SyncContext::throwError(params.m_name + ": virtual source cannot be instantiated");
+        }
+        return source;
+    }
+
     const SourceRegistry &registry(getSourceRegistry());
     BOOST_FOREACH(const RegisterSyncSource *sourceInfos, registry) {
         SyncSource *source = sourceInfos->m_create(params);
@@ -334,9 +351,9 @@ SyncSource *SyncSource::createSource(const SyncSourceParams &params, bool error)
 SyncSource *SyncSource::createTestingSource(const string &name, const string &type, bool error,
                                             const char *prefix)
 {
-    SyncConfig config("testing");
+    SyncConfig config("testing@client-test");
     SyncSourceNodes nodes = config.getSyncSourceNodes(name);
-    SyncSourceParams params(name, nodes, "");
+    SyncSourceParams params(name, nodes);
     PersistentSyncSourceConfig sourceconfig(name, nodes);
     sourceconfig.setSourceType(type);
     if (prefix) {
@@ -345,6 +362,65 @@ SyncSource *SyncSource::createTestingSource(const string &name, const string &ty
     return createSource(params, error);
 }
 
+VirtualSyncSource::VirtualSyncSource(const SyncSourceParams &params, SyncConfig *config) :
+    DummySyncSource(params)
+{
+    if (config) {
+        BOOST_FOREACH(std::string name, getMappedSources()) {
+            SyncSourceNodes source = config->getSyncSourceNodes(name);
+            SyncSourceParams params(name, source);
+            boost::shared_ptr<SyncSource> syncSource(createSource(params, true, config));
+            m_sources.push_back(syncSource);
+        }
+    }
+}
+
+void VirtualSyncSource::open()
+{
+    getDataTypeSupport();
+    BOOST_FOREACH(boost::shared_ptr<SyncSource> &source, m_sources) {
+        source->open();
+    }
+}
+
+void VirtualSyncSource::close()
+{
+    BOOST_FOREACH(boost::shared_ptr<SyncSource> &source, m_sources) {
+        source->close();
+    }
+}
+
+std::vector<std::string> VirtualSyncSource::getMappedSources()
+{
+    std::string evoSyncSource = getDatabaseID();
+    std::vector<std::string> mappedSources = unescapeJoinedString (evoSyncSource, ',');
+    return mappedSources;
+}
+
+std::string VirtualSyncSource::getDataTypeSupport()
+{
+    string datatypes;
+    SourceType sourceType = getSourceType();
+    string type = sourceType.m_format;
+
+    datatypes = getDataTypeSupport(type, sourceType.m_forceFormat);
+    return datatypes;
+}
+
+SyncSource::Databases VirtualSyncSource::getDatabases()
+{
+    SyncSource::Databases dbs;
+    BOOST_FOREACH (boost::shared_ptr<SyncSource> &source, m_sources) {
+        SyncSource::Databases sub = source->getDatabases();
+        if (sub.empty()) {
+            return dbs;
+        }
+    }
+    Database db ("calendar+todo", "");
+    dbs.push_back (db);
+    return dbs;
+}
+
 void SyncSourceSession::init(SyncSource::Operations &ops)
 {
     ops.m_startDataRead = boost::bind(&SyncSourceSession::startDataRead, this, _1, _2);
@@ -440,7 +516,7 @@ void SyncSourceSerialize::getSynthesisInfo(SynthesisInfo &info,
         info.m_datatypes =
             "        <use datatype='vCard21' mode='rw'/>\n"
             "        <use datatype='vCard30' mode='rw' preferred='yes'/>\n";
-    } else if (type == "text/x-calendar") {
+    } else if (type == "text/x-calendar" || type == "text/x-vcalendar") {
         info.m_native = "vCalendar10";
         info.m_fieldlist = "calendar";
         info.m_profile = "\"vCalendar\", 1";
@@ -473,43 +549,55 @@ void SyncSourceSerialize::getSynthesisInfo(SynthesisInfo &info,
     if (!sourceType.m_format.empty()) {
         type = sourceType.m_format;
     }
+    info.m_datatypes = getDataTypeSupport(type, sourceType.m_forceFormat);
+}
+
+std::string SyncSourceBase::getDataTypeSupport(const std::string &type,
+                                               bool forceFormat)
+{
+    std::string datatypes;
 
     if (type == "text/x-vcard:2.1" || type == "text/x-vcard") {
-        info.m_datatypes =
+        datatypes =
             "        <use datatype='vCard21' mode='rw' preferred='yes'/>\n";
-        if (!sourceType.m_forceFormat) {
-            info.m_datatypes +=
+        if (!forceFormat) {
+            datatypes +=
                 "        <use datatype='vCard30' mode='rw'/>\n";
         }
     } else if (type == "text/vcard:3.0" || type == "text/vcard") {
-        info.m_datatypes =
+        datatypes =
             "        <use datatype='vCard30' mode='rw' preferred='yes'/>\n";
-        if (!sourceType.m_forceFormat) {
-            info.m_datatypes +=
+        if (!forceFormat) {
+            datatypes +=
                 "        <use datatype='vCard21' mode='rw'/>\n";
         }
-    } else if (type == "text/x-vcalendar:1.0" || type == "text/x-vcalendar") {
-        info.m_datatypes =
+    } else if (type == "text/x-vcalendar:1.0" || type == "text/x-vcalendar"
+             || type == "text/x-calendar:1.0" || type == "text/x-calendar") {
+        datatypes =
             "        <use datatype='vcalendar10' mode='rw' preferred='yes'/>\n";
-        if (!sourceType.m_forceFormat) {
-            info.m_datatypes +=
+        if (!forceFormat) {
+            datatypes +=
                 "        <use datatype='icalendar20' mode='rw'/>\n";
         }
     } else if (type == "text/calendar:2.0" || type == "text/calendar") {
-        info.m_datatypes =
+        datatypes =
             "        <use datatype='icalendar20' mode='rw' preferred='yes'/>\n";
-        if (!sourceType.m_forceFormat) {
-            info.m_datatypes +=
+        if (!forceFormat) {
+            datatypes +=
                 "        <use datatype='vcalendar10' mode='rw'/>\n";
         }
     } else if (type == "text/plain:1.0" || type == "text/plain") {
         // note10 are the same as note11, so ignore force format
-        info.m_datatypes =
+        datatypes =
             "        <use datatype='note10' mode='rw' preferred='yes'/>\n"
             "        <use datatype='note11' mode='rw'/>\n";
+    } else if (type.empty()) {
+        throwError("no MIME type configured");
     } else {
         throwError(string("configured MIME type not supported: ") + type);
     }
+
+    return datatypes;
 }
 
 sysync::TSyError SyncSourceSerialize::readItemAsKey(sysync::cItemID aID, sysync::KeyH aItemKey)
@@ -545,66 +633,199 @@ void SyncSourceSerialize::init(SyncSource::Operations &ops)
                                         this, _1, _2, _3);
 }
 
-
-void SyncSourceRevisions::backupData(const string &dir, ConfigNode &node, BackupReport &report)
+/**
+ * Mapping from Hash() value to file.
+ */
+class ItemCache
 {
-    RevisionMap_t revisions;
-    listAllItems(revisions);
+public:
+#ifdef USE_SHA256
+    typedef std::string Hash_t;
+    Hash_t hashFunc(const std::string &data) { return SHA_256(data); }
+#else
+    typedef unsigned long Hash_t;
+    Hash_t hashFunc(const std::string &data) { return Hash(data); }
+#endif
+    typedef unsigned long Counter_t;
+
+    /** mark the algorithm used for the hash via different suffices */
+    static const char *m_hashSuffix;
+
+    /**
+     * Collect information about stored hashes. Provides
+     * access to file name via hash.
+     *
+     * If no hashes were written (as in an old SyncEvoltion
+     * version), we could read the files to recreate the
+     * hashes. This is not done because it won't occur
+     * often enough.
+     *
+     * Hashes are also not verified. Users should better
+     * not edit them or file contents...
+     *
+     * @param oldBackup     existing backup to read; may be empty
+     */
+    void init(const SyncSource::Operations::ConstBackupInfo &oldBackup)
+    {
+        m_hash2counter.clear();
+        m_dirname = oldBackup.m_dirname;
+        if (m_dirname.empty() || !oldBackup.m_node) {
+            return;
+        }
+
+        long numitems;
+        if (!oldBackup.m_node->getProperty("numitems", numitems)) {
+            return;
+        }
+        for (long counter = 1; counter <= numitems; counter++) {
+            stringstream key;
+            key << counter << m_hashSuffix;
+            Hash_t hash;
+            if (oldBackup.m_node->getProperty(key.str(), hash)) {
+                m_hash2counter[hash] = counter;
+            }
+        }
+    }
+
+    /**
+     * create file name for a specific hash, empty if no such hash
+     */
+    string getFilename(Hash_t hash)
+    {
+        Map_t::const_iterator it = m_hash2counter.find(hash);
+        if (it != m_hash2counter.end()) {
+            stringstream dirname;
+            dirname << m_dirname << "/" << it->second;
+            return dirname.str();
+        } else {
+            return "";
+        }
+    }
+
+private:
+    typedef std::map<Hash_t, Counter_t> Map_t;
+    Map_t m_hash2counter;
+    string m_dirname;
+};
+
+const char *ItemCache::m_hashSuffix =
+#ifdef USE_SHA256
+    "-sha256"
+#else
+    "-hash"
+#endif
+;
+
+void SyncSourceRevisions::initRevisions()
+{
+    if (!m_revisionsSet) {
+        listAllItems(m_revisions);
+        m_revisionsSet = true;
+    }
+}
+
+
+void SyncSourceRevisions::backupData(const SyncSource::Operations::ConstBackupInfo &oldBackup,
+                                     const SyncSource::Operations::BackupInfo &newBackup,
+                                     BackupReport &report)
+{
+    ItemCache cache;
+    cache.init(oldBackup);
+
+    bool startOfSync = newBackup.m_mode == SyncSource::Operations::BackupInfo::BACKUP_BEFORE;
+    RevisionMap_t buffer;
+    RevisionMap_t *revisions;
+    if (startOfSync) {
+        initRevisions();
+        revisions = &m_revisions;
+    } else {
+        listAllItems(buffer);
+        revisions = &buffer;
+    }
 
     unsigned long counter = 1;
     string item;
     errno = 0;
-    BOOST_FOREACH(const StringPair &mapping, revisions) {
+    BOOST_FOREACH(const StringPair &mapping, *revisions) {
         const string &uid = mapping.first;
         const string &rev = mapping.second;
         m_raw->readItemRaw(uid, item);
 
         stringstream filename;
-        filename << dir << "/" << counter;
+        filename << newBackup.m_dirname << "/" << counter;
 
-        ofstream out(filename.str().c_str());
-        out.write(item.c_str(), item.size());
-        out.close();
-        if (out.fail()) {
-            throwError(string("error writing ") + filename.str() + ": " + strerror(errno));
+        ItemCache::Hash_t hash = cache.hashFunc(item);
+        string oldfilename = cache.getFilename(hash);
+        if (!oldfilename.empty()) {
+            // found old file with same content, reuse it via hardlink
+            if (link(oldfilename.c_str(), filename.str().c_str())) {
+                // Hard linking failed. Record this, then continue
+                // by ignoring the old file.
+                SE_LOG_DEBUG(NULL, NULL, "hard linking old %s new %s: %s",
+                             oldfilename.c_str(),
+                             filename.str().c_str(),
+                             strerror(errno));
+                oldfilename.clear();
+            }
+        }
+
+        if (oldfilename.empty()) {
+            // write new file instead of reusing old one
+            ofstream out(filename.str().c_str());
+            out.write(item.c_str(), item.size());
+            out.close();
+            if (out.fail()) {
+                throwError(string("error writing ") + filename.str() + ": " + strerror(errno));
+            }
         }
 
         stringstream key;
         key << counter << "-uid";
-        node.setProperty(key.str(), uid);
+        newBackup.m_node->setProperty(key.str(), uid);
+        // clear() does not remove the existing content, which was
+        // intended here. This should have been key.str(""). As a
+        // result, keys for -rev are longer than intended because they
+        // start with the -uid part. We cannot change it now, because
+        // that would break compatibility with nodes that use the
+        // older, longer keys for -rev.
         key.clear();
         key << counter << "-rev";
-        node.setProperty(key.str(), rev);
+        newBackup.m_node->setProperty(key.str(), rev);
+        key.str("");
+        key << counter << ItemCache::m_hashSuffix;
+        newBackup.m_node->setProperty(key.str(), hash);
 
         counter++;
     }
 
     stringstream value;
     value << counter - 1;
-    node.setProperty("numitems", value.str());
-    node.flush();
+    newBackup.m_node->setProperty("numitems", value.str());
+    newBackup.m_node->flush();
 
     report.setNumItems(counter - 1);
 }
 
-void SyncSourceRevisions::restoreData(const string &dir, const ConfigNode &node, bool dryrun, SyncSourceReport &report)
+void SyncSourceRevisions::restoreData(const SyncSource::Operations::ConstBackupInfo &oldBackup,
+                                      bool dryrun,
+                                      SyncSourceReport &report)
 {
     RevisionMap_t revisions;
     listAllItems(revisions);
 
     long numitems;
     string strval;
-    strval = node.readProperty("numitems");
+    strval = oldBackup.m_node->readProperty("numitems");
     stringstream stream(strval);
     stream >> numitems;
 
     for (long counter = 1; counter <= numitems; counter++) {
         stringstream key;
         key << counter << "-uid";
-        string uid = node.readProperty(key.str());
+        string uid = oldBackup.m_node->readProperty(key.str());
         key.clear();
         key << counter << "-rev";
-        string rev = node.readProperty(key.str());
+        string rev = oldBackup.m_node->readProperty(key.str());
         RevisionMap_t::iterator it = revisions.find(uid);
         report.incrementItemStat(report.ITEM_LOCAL,
                                  report.ITEM_ANY,
@@ -616,7 +837,7 @@ void SyncSourceRevisions::restoreData(const string &dir, const ConfigNode &node,
         } else {
             // add or update, so need item
             stringstream filename;
-            filename << dir << "/" << counter;
+            filename << oldBackup.m_dirname << "/" << counter;
             string data;
             if (!ReadFile(filename.str(), data)) {
                 throwError(StringPrintf("restoring %s from %s failed: could not read file",
@@ -682,10 +903,9 @@ void SyncSourceRevisions::restoreData(const string &dir, const ConfigNode &node,
 
 void SyncSourceRevisions::detectChanges(ConfigNode &trackingNode)
 {
-    RevisionMap_t revisions;
-    listAllItems(revisions);
+    initRevisions();
 
-    BOOST_FOREACH(const StringPair &mapping, revisions) {
+    BOOST_FOREACH(const StringPair &mapping, m_revisions) {
         const string &uid = mapping.first;
         const string &revision = mapping.second;
 
@@ -705,7 +925,7 @@ void SyncSourceRevisions::detectChanges(ConfigNode &trackingNode)
     }
 
     // clear information about all items that we recognized as deleted
-    map<string, string> props;
+    ConfigProps props;
     trackingNode.readProperties(props);
 
     BOOST_FOREACH(const StringPair &mapping, props) {
@@ -762,13 +982,14 @@ void SyncSourceRevisions::init(SyncSourceRaw *raw,
     m_del = del;
     m_modTimeStamp = 0;
     m_revisionAccuracySeconds = granularity;
+    m_revisionsSet = false;
     if (raw) {
         ops.m_backupData = boost::bind(&SyncSourceRevisions::backupData,
                                        this, _1, _2, _3);
     }
     if (raw && del) {
         ops.m_restoreData = boost::bind(&SyncSourceRevisions::restoreData,
-                                        this, _1, _2, _3, _4);
+                                        this, _1, _2, _3);
     }
     ops.m_endSession.push_back(boost::bind(&SyncSourceRevisions::sleepSinceModification,
                                            this));
@@ -861,5 +1082,203 @@ void SyncSourceLogging::init(const std::list<std::string> &fields,
                                    this, _1, ops.m_deleteItem);
 }
 
+sysync::TSyError SyncSourceAdmin::loadAdminData(const char *aLocDB,
+                                                const char *aRemDB,
+                                                char **adminData)
+{
+    std::string data = m_configNode->readProperty(m_adminPropertyName);
+    *adminData = StrAlloc(SafeConfigNode::unescape(data).c_str());
+    resetMap();
+    return sysync::LOCERR_OK;
+}
+
+sysync::TSyError SyncSourceAdmin::saveAdminData(const char *adminData)
+{
+    m_configNode->setProperty(m_adminPropertyName,
+                              SafeConfigNode::escape(adminData, false, false));
+
+    // Flush here, because some calls to saveAdminData() happend
+    // after SyncSourceAdmin::flush() (= session end).
+    m_configNode->flush();
+    return sysync::LOCERR_OK;
+}
+
+bool SyncSourceAdmin::readNextMapItem(sysync::MapID mID, bool aFirst)
+{
+    if (aFirst) {
+        resetMap();
+    }
+    if (m_mappingIterator != m_mapping.end()) {
+        entry2mapid(m_mappingIterator->first, m_mappingIterator->second, mID);
+        ++m_mappingIterator;
+        return true;
+    } else {
+        return false;
+    }
+}
+
+sysync::TSyError SyncSourceAdmin::insertMapItem(sysync::cMapID mID)
+{
+    string key, value;
+    mapid2entry(mID, key, value);
+
+#if 0
+    StringMap::iterator it = m_mapping.find(key);
+    if (it != m_mapping.end()) {
+        // error, exists already
+        return sysync::DB_Forbidden;
+    } else {
+        m_mapping[key] = value;
+        return sysync::LOCERR_OK;
+    }
+#else
+    m_mapping[key] = value;
+    m_mappingNode->clear();
+    m_mappingNode->writeProperties(m_mapping);
+    m_mappingNode->flush();
+    return sysync::LOCERR_OK;
+#endif
+}
+
+sysync::TSyError SyncSourceAdmin::updateMapItem(sysync::cMapID mID)
+{
+    string key, value;
+    mapid2entry(mID, key, value);
+
+    StringMap::iterator it = m_mapping.find(key);
+    if (it == m_mapping.end()) {
+        // error, does not exist
+        return sysync::DB_Forbidden;
+    } else {
+        m_mapping[key] = value;
+        m_mappingNode->clear();
+        m_mappingNode->writeProperties(m_mapping);
+        m_mappingNode->flush();
+        return sysync::LOCERR_OK;
+    }
+}
+
+sysync::TSyError SyncSourceAdmin::deleteMapItem(sysync::cMapID mID)
+{
+    string key, value;
+    mapid2entry(mID, key, value);
+
+    StringMap::iterator it = m_mapping.find(key);
+    if (it == m_mapping.end()) {
+        // error, does not exist
+        return sysync::DB_Forbidden;
+    } else {
+        m_mapping.erase(it);
+        m_mappingNode->clear();
+        m_mappingNode->writeProperties(m_mapping);
+        m_mappingNode->flush();
+        return sysync::LOCERR_OK;
+    }
+}
+
+void SyncSourceAdmin::flush()
+{
+    m_configNode->flush();
+    if (m_mappingLoaded) {
+        m_mappingNode->clear();
+        m_mappingNode->writeProperties(m_mapping);
+        m_mappingNode->flush();
+    }
+}
+
+void SyncSourceAdmin::resetMap()
+{
+    m_mapping.clear();
+    m_mappingNode->readProperties(m_mapping);
+    m_mappingIterator = m_mapping.begin();
+    m_mappingLoaded = true;
+}
+
+
+void SyncSourceAdmin::mapid2entry(sysync::cMapID mID, string &key, string &value)
+{
+    key = StringPrintf ("%s-%x",
+                         SafeConfigNode::escape(mID->localID ? mID->localID : "", true, false).c_str(),
+                         mID->ident);
+    if (mID->remoteID && mID->remoteID[0]) {
+        value = StringPrintf("%s %x",
+                             SafeConfigNode::escape(mID->remoteID ? mID->remoteID : "", true, false).c_str(),
+                             mID->flags);
+    } else {
+        value = StringPrintf("%x", mID->flags);
+    }
+}
+
+void SyncSourceAdmin::entry2mapid(const string &key, const string &value, sysync::MapID mID)
+{
+    size_t found = key.rfind('-');
+    mID->localID = StrAlloc(SafeConfigNode::unescape(key.substr(0,found)).c_str());
+    if (found != key.npos) {
+        mID->ident =  strtol(key.substr(found+1).c_str(), NULL, 16);
+    } else {
+        mID->ident = 0;
+    }
+    std::vector< std::string > tokens;
+    boost::split(tokens, value, boost::is_from_range(' ', ' '));
+    if (tokens.size() >= 2) {
+        // if branch from mapid2entry above
+        mID->remoteID = StrAlloc(SafeConfigNode::unescape(tokens[0]).c_str());
+        mID->flags = strtol(tokens[1].c_str(), NULL, 16);
+    } else {
+        // else branch from above
+        mID->remoteID = NULL;
+        mID->flags = strtol(tokens[0].c_str(), NULL, 16);
+    }
+}
+
+void SyncSourceAdmin::init(SyncSource::Operations &ops,
+                           const boost::shared_ptr<ConfigNode> &config,
+                           const std::string adminPropertyName,
+                           const boost::shared_ptr<ConfigNode> &mapping)
+{
+    m_configNode = config;
+    m_adminPropertyName = adminPropertyName;
+    m_mappingNode = mapping;
+    m_mappingLoaded = false;
+
+    ops.m_loadAdminData = boost::bind(&SyncSourceAdmin::loadAdminData,
+                                      this, _1, _2, _3);
+    ops.m_saveAdminData = boost::bind(&SyncSourceAdmin::saveAdminData,
+                                      this, _1);
+    ops.m_readNextMapItem = boost::bind(&SyncSourceAdmin::readNextMapItem,
+                                        this, _1, _2);
+    ops.m_insertMapItem = boost::bind(&SyncSourceAdmin::insertMapItem,
+                                      this, _1);
+    ops.m_updateMapItem = boost::bind(&SyncSourceAdmin::updateMapItem,
+                                      this, _1);
+    ops.m_deleteMapItem = boost::bind(&SyncSourceAdmin::deleteMapItem,
+                                      this, _1);
+    ops.m_endSession.push_back(boost::bind(&SyncSourceAdmin::flush,
+                                           this));
+}
+
+void SyncSourceAdmin::init(SyncSource::Operations &ops,
+                           SyncSource *source)
+{
+    init(ops,
+         source->getProperties(true),
+         SourceAdminDataName,
+         source->getServerNode());
+}
+
+void SyncSourceBlob::init(SyncSource::Operations &ops,
+                          const std::string &dir)
+{
+    m_blob.Init(getSynthesisAPI(),
+                getName(),
+                dir, "", "", "");
+    ops.m_readBlob = boost::bind(&SyncSourceBlob::readBlob, this,
+                                 _1, _2, _3, _4, _5, _6, _7);
+    ops.m_writeBlob = boost::bind(&SyncSourceBlob::writeBlob, this,
+                                  _1, _2, _3, _4, _5, _6, _7);
+    ops.m_deleteBlob = boost::bind(&SyncSourceBlob::deleteBlob, this,
+                                   _1, _2);
+}
 
 SE_END_CXX
+
