@@ -27,6 +27,7 @@
 
 #include <synthesis/sync_declarations.h>
 #include <synthesis/syerror.h>
+#include <synthesis/blobs.h>
 
 #include <boost/function.hpp>
 
@@ -44,40 +45,15 @@ struct SyncSourceParams {
     /**
      * @param    name        the name needed by SyncSource
      * @param    nodes       a set of config nodes to be used by this source
-     * @param    changeId    a unique string constructed from an ID for SyncEvolution
-     *                       and the URL/database we synchronize against; can be
-     *                       used to do change tracking for that combination of
-     *                       peers
      */
     SyncSourceParams(const string &name,
-                     const SyncSourceNodes &nodes,
-                     const string &changeId) :
-    m_name(name),
-        m_nodes(nodes),
-        m_changeId(stripChangeId(changeId))
+                     const SyncSourceNodes &nodes = SyncSourceNodes()) :
+        m_name(name),
+        m_nodes(nodes)
     {}
 
-    const string m_name;
-    const SyncSourceNodes m_nodes;
-    const string m_changeId;
-
-    /** remove special characters from change ID */
-    static string stripChangeId(const string changeId) {
-        string strippedChangeId = changeId;
-        size_t offset = 0;
-        while (offset < strippedChangeId.size()) {
-            switch (strippedChangeId[offset]) {
-            case ':':
-            case '/':
-            case '\\':
-                strippedChangeId.erase(offset, 1);
-                break;
-            default:
-                offset++;
-            }
-        }
-        return strippedChangeId;
-    }
+    string m_name;
+    SyncSourceNodes m_nodes;
 };
 
 /**
@@ -395,6 +371,11 @@ struct ClientTestConfig{
      */
     const char *type;
 
+    /**
+     * a list of sub configs separated via , if this is a super datastore
+     */
+    const char *subConfigs;
+
     /**
      * TRUE if the source supports recovery from an interrupted
      * synchronization. Enables the Client::Sync::*::Retry group
@@ -658,6 +639,12 @@ class SyncSourceBase : public Logger {
      */
     virtual SDKInterface *getSynthesisAPI() const = 0;
 
+    /**
+     * Prepare the sync source for usage inside a SyncML server.  To
+     * be called directly after creating the source, if at all.
+     */
+    virtual void enableServerMode() = 0;
+    virtual bool serverModeEnabled() const = 0;
 
  protected:
     struct SynthesisInfo {
@@ -666,6 +653,13 @@ class SyncSourceBase : public Logger {
          * leave empty when acessing the field list directly
          */
         std::string m_profile;
+
+        /**
+         * the second parameter for MAKE/PARSETEXTWITHPROFILE
+         * which specifies a remote rule to be applied when
+         * converting to and from the backend
+         */
+        std::string m_backendRule;
     
         /** list of supported datatypes in "<use .../>" format */
         std::string m_datatypes;
@@ -707,6 +701,21 @@ class SyncSourceBase : public Logger {
      */
     virtual void getSynthesisInfo(SynthesisInfo &info,
                                   XMLConfigFragments &fragments) = 0;
+
+    /**
+     * utility code: creates Synthesis <use datatype=...>
+     * statements, using the predefined vCard21/vCard30/vcalendar10/icalendar20
+     * types. Throws an error if no suitable result can be returned (empty or invalid type)
+     *
+     * @param type         the format specifier as used in SyncEvolution configs, with and without version
+     *                     (text/x-vcard:2.1, text/x-vcard, text/x-vcalendar, text/calendar, text/plain, ...);
+     *                     see SourceType::m_format
+     * @param forceFormat  if true, then don't allow alternative formats (like vCard 3.0 in addition to 2.1);
+     *                     see SourceType::m_force
+     * @return generated XML fragment
+     */
+    std::string getDataTypeSupport(const std::string &type,
+                                   bool forceFormat);
 };
 
 /**
@@ -727,7 +736,8 @@ class SyncSource : virtual public SyncSourceBase, public SyncSourceConfig, publi
  public:
     SyncSource(const SyncSourceParams &params) :
         SyncSourceConfig(params.m_name, params.m_nodes),
-        m_numDeleted(0)
+        m_numDeleted(0),
+        m_forceSlowSync(false)
         {
         }
     virtual ~SyncSource() {}
@@ -762,8 +772,17 @@ class SyncSource : virtual public SyncSourceBase, public SyncSourceConfig, publi
     /**
      * Actually opens the data source specified in the constructor,
      * will throw the normal exceptions if that fails. Should
-     * not modify the state of the sync source: that can be deferred
-     * until the server is also ready and beginSync() is called.
+     * not modify the state of the sync source.
+     *
+     * The expectation is that this call is fairly light-weight, but
+     * does enough checking to determine whether the source is
+     * usable. More expensive operations (like determining changes)
+     * should be done in the m_startDataRead callback (bound to
+     * beginSync() in some of the utility classes).
+     *
+     * In clients, it will be called for all sources before
+     * the sync starts. In servers, it is called for each source once
+     * the client asks for it, but not sooner.
      */
     virtual void open() = 0;
 
@@ -782,22 +801,77 @@ class SyncSource : virtual public SyncSourceBase, public SyncSourceConfig, publi
      */
     struct Operations {
         /**
-         * Dump all data from source unmodified into the given directory.
-         * The ConfigNode can be used to store meta information needed for
-         * restoring that state. Both directory and node are empty.
+         * The caller determines where item data is stored (m_dirname)
+         * and where meta information about them (m_node). The callee
+         * then can use both arbitrarily. As an additional hint,
+         * m_mode specifies why and when the backup is made, which
+         * is useful to determine whether information can be reused.
+         */
+        struct BackupInfo {
+            enum Mode {
+                BACKUP_BEFORE,   /**< directly at start of sync */
+                BACKUP_AFTER,    /**< directly after sync */
+                BACKUP_OTHER
+            } m_mode;
+            string m_dirname;
+            boost::shared_ptr<ConfigNode> m_node;
+            BackupInfo() {}
+            BackupInfo(Mode mode,
+                       const string &dirname,
+                       const boost::shared_ptr<ConfigNode> &node) :
+                m_mode(mode),
+                m_dirname(dirname),
+                m_node(node)
+            {}
+        };
+        struct ConstBackupInfo {
+            BackupInfo::Mode m_mode;
+            string m_dirname;
+            boost::shared_ptr<const ConfigNode> m_node;
+            ConstBackupInfo() {}
+            ConstBackupInfo(BackupInfo::Mode mode,
+                            const string &dirname,
+                            const boost::shared_ptr<const ConfigNode> &node) :
+                m_mode(mode),
+                m_dirname(dirname),
+                m_node(node)
+            {}
+        };
+
+        /**
+         * Dump all data from source unmodified into the given backup location.
          * Information about the created backup is added to the
          * report.
          *
-         * Required for the backup/restore functionality in SyncEvolution,
-         * not for syncing itself.
+         * Required for the backup/restore functionality in
+         * SyncEvolution, not for syncing itself. But typically it is
+         * called before syncing (can be turned off by users), so
+         * implementations can reuse the information gathered while
+         * making a backup in later operations.
+         *
+         * @param previous     the most recent backup, empty m_dirname if none
+         * @param next         the backup which is to be created, directory and node are empty
+         * @param report       to be filled with information about backup (number of items, etc.)
          */
-        typedef void (BackupData_t)(const string &dirname, ConfigNode &node, BackupReport &report);
+        typedef void (BackupData_t)(const ConstBackupInfo &oldBackup,
+                                    const BackupInfo &newBackup,
+                                    BackupReport &report);
         boost::function<BackupData_t> m_backupData;
 
         /**
          * Restore database from data stored in backupData().
+         * If possible don't touch items which are the same as in the
+         * backup, to mimimize impact on future incremental syncs.
+         *
+         * @param oldBackup    the backup which is to be restored
+         * @param dryrun       pretend to restore and fill in report, without
+         *                     actually touching backend data
+         * @param report       to be filled with information about restore
+         *                     (number of total items and changes)
          */
-        typedef void (RestoreData_t)(const string &dirname, const ConfigNode &node, bool dryrun, SyncSourceReport &report);
+        typedef void (RestoreData_t)(const ConstBackupInfo &oldBackup,
+                                     bool dryrun,
+                                     SyncSourceReport &report);
         boost::function<RestoreData_t> m_restoreData;
 
         /**
@@ -816,6 +890,20 @@ class SyncSource : virtual public SyncSourceBase, public SyncSourceConfig, publi
         typedef void (CheckStatus_t)(SyncSourceReport &local);
         boost::function<CheckStatus_t> m_checkStatus;
 
+        /**
+         * A quick check whether the source currently has data.
+         *
+         * If this cannot be determined easily, don't provide the
+         * operation. The information is currently only used to
+         * determine whether a slow sync should be allowed. If
+         * the operation is not provided, the assumption is that
+         * there is local data, which disables the "allow slow
+         * sync for empty databases" heuristic and forces the user
+         * to choose.
+         */
+        typedef bool (IsEmpty_t)();
+        boost::function<IsEmpty_t> m_isEmpty;
+
         /**
          * Synthesis DB API callbacks. For documentation see the
          * Synthesis API specification (PDF and/or sync_dbapi.h).
@@ -824,14 +912,20 @@ class SyncSource : virtual public SyncSourceBase, public SyncSourceConfig, publi
          * to be part of a sync session.
          */
         /**@{*/
+        typedef void (Callback_t)();
+        typedef boost::function<Callback_t> CallbackFunctor_t;
+        typedef std::list<CallbackFunctor_t> Callbacks_t;
+
+        /** all of these functions will be called before accessing
+            the source's data for the first time, i.e., before m_startDataRead */
+        Callbacks_t m_startAccess;
+
         typedef sysync::TSyError (StartDataRead_t)(const char *lastToken, const char *resumeToken);
         boost::function<StartDataRead_t> m_startDataRead;
 
-        typedef void (Callback_t)();
-        typedef boost::function<Callback_t> CallbackFunctor_t;
         /** all of these functions will be called directly after
             m_startDataRead() returned successfully */
-        std::list<CallbackFunctor_t> m_startSession;
+        Callbacks_t m_startSession;
 
         typedef sysync::TSyError (EndDataRead_t)();
         boost::function<EndDataRead_t> m_endDataRead;
@@ -864,8 +958,64 @@ class SyncSource : virtual public SyncSourceBase, public SyncSourceConfig, publi
         typedef sysync::TSyError (DeleteItem_t)(sysync::cItemID aID);
         boost::function<DeleteItem_t> m_deleteItem;
         /**@}*/
+
+
+        /**
+         * Synthesis administration callbacks. For documentation see the
+         * Synthesis API specification (PDF and/or sync_dbapi.h).
+         *
+         * Implementing this is *optional* in clients. In the Synthesis client
+         * engine, the "binfiles" module provides these calls without SyncEvolution
+         * doing anything.
+         *
+         * In the Synthesis server engine, the
+         * SyncSource::enableServerMode() call must install an
+         * implementation, like the one from SyncSourceAdmin.
+         */
+        /**@{*/
+        typedef sysync::TSyError (LoadAdminData_t)(const char *aLocDB,
+                                                   const char *aRemDB,
+                                                   char **adminData);
+        boost::function<LoadAdminData_t> m_loadAdminData;
+
+        typedef sysync::TSyError (SaveAdminData_t)(const char *adminData);
+        boost::function<SaveAdminData_t> m_saveAdminData;
+
+        typedef bool (ReadNextMapItem_t)(sysync::MapID mID, bool aFirst);
+        boost::function<ReadNextMapItem_t> m_readNextMapItem;
+
+        typedef sysync::TSyError (InsertMapItem_t)(sysync::cMapID mID);
+        boost::function<InsertMapItem_t> m_insertMapItem;
+
+        typedef sysync::TSyError (UpdateMapItem_t)(sysync::cMapID mID);
+        boost::function<UpdateMapItem_t> m_updateMapItem;
+
+        typedef sysync::TSyError (DeleteMapItem_t)(sysync::cMapID mID);
+        boost::function<DeleteMapItem_t> m_deleteMapItem;
+
+        typedef sysync::TSyError (ReadBlob_t)(sysync::cItemID aID, const char *aBlobID,
+                                              void **aBlkPtr, size_t *aBlkSize,
+                                              size_t *aTotSize,
+                                              bool aFirst, bool *aLast);
+        boost::function<ReadBlob_t> m_readBlob;
+
+        typedef sysync::TSyError (WriteBlob_t)(sysync::cItemID aID, const char *aBlobID,
+                                               void *aBlkPtr, size_t aBlkSize,
+                                               size_t aTotSize,
+                                               bool aFirst, bool aLast);
+        boost::function<WriteBlob_t> m_writeBlob;
+
+        typedef sysync::TSyError (DeleteBlob_t)(sysync::cItemID aID, const char *aBlobID);
+        boost::function<DeleteBlob_t> m_deleteBlob;
+        /**@}*/
     };
     const Operations &getOperations() { return m_operations; }
+
+    /**
+     * outside users of the source are only allowed to add callbacks,
+     * not overwrite arbitrary operations
+     */
+    void addCallback(Operations::CallbackFunctor_t callback, Operations::Callbacks_t Operations::* where) { (m_operations.*where).push_back(callback); }
         
     /**
      * closes the data source so that it can be reopened
@@ -898,10 +1048,12 @@ class SyncSource : virtual public SyncSourceBase, public SyncSourceConfig, publi
      * source type specified in the params.m_nodes.m_configNode
      *
      * @param error    throw a runtime error describing what the problem is if no matching source is found
-     * @return NULL if no source can handle the given type
+     * @param config   optional, needed for intantiating virtual sources
+     * @return valid instance, NULL if no source can handle the given type (only when error==false)
      */
     static SyncSource *createSource(const SyncSourceParams &params,
-                                    bool error = true);
+                                    bool error = true,
+                                    SyncConfig *config = NULL);
 
     /**
      * Factory function for a SyncSource with the given name
@@ -932,12 +1084,28 @@ class SyncSource : virtual public SyncSourceBase, public SyncSourceConfig, publi
      */
     static string backendsDebug();
 
+    /**
+     * Mime type a backend communicates with the remote peer by default,
+     * this is used to alert the remote peer in SAN during server alerted sync.
+     */
+    virtual const char *getPeerMimeType() const =0;
+
     /* implementation of SyncSourceBase */
     virtual const char * getName() const { return SyncSourceConfig::getName(); }
     virtual long getNumDeleted() const { return m_numDeleted; }
     virtual void setNumDeleted(long num) { m_numDeleted = num; }
     virtual void incrementNumDeleted() { m_numDeleted++; }
 
+    /**
+     * Set to true in SyncContext::initSAN() when a SyncML server has
+     * to force a client into slow sync mode. This is necessary because
+     * the server cannot request that mode (missing in the standard).
+     * Forcing the slow sync mode is done via a FORCESLOWSYNC() macro
+     * call in an <alertscript>.
+     */
+    void setForceSlowSync(bool forceSlowSync) { m_forceSlowSync = forceSlowSync; }
+    bool getForceSlowSync() const { return m_forceSlowSync; }
+
  protected:
     Operations m_operations;
 
@@ -951,6 +1119,8 @@ class SyncSource : virtual public SyncSourceBase, public SyncSourceConfig, publi
      */
     long m_numDeleted;
 
+    bool m_forceSlowSync;
+
     /**
      * Interface pointer for this sync source, allocated for us by the
      * Synthesis engine and registered here by
@@ -961,6 +1131,72 @@ class SyncSource : virtual public SyncSourceBase, public SyncSourceConfig, publi
     std::vector<sysync::SDK_InterfaceType *> m_synthesisAPI;
 };
 
+/**
+ * A SyncSource with no pure virtual functions.
+ */
+class DummySyncSource : public SyncSource
+{
+ public:
+    DummySyncSource(const SyncSourceParams &params) :
+       SyncSource(params) {}
+
+    DummySyncSource(const std::string &name) :
+       SyncSource(SyncSourceParams(name)) {}
+
+    virtual Databases getDatabases() { return Databases(); }
+    virtual void open() {}
+    virtual void close() {}
+    virtual void getSynthesisInfo(SynthesisInfo &info,
+                                  XMLConfigFragments &fragments) {}
+    virtual void enableServerMode() {}
+    virtual bool serverModeEnabled() const { return false; }
+    virtual const char *getPeerMimeType() const {return "";} 
+};
+
+/**
+ * A special source which combines one or more real sources.
+ * Most of the special handling for that is in SyncContext.cpp.
+ *
+ * This class can be instantiated, opened and closed if and only if
+ * the underlying sources also support that.
+ */
+class VirtualSyncSource : public DummySyncSource 
+{
+    std::vector< boost::shared_ptr<SyncSource> > m_sources;
+
+public:
+    /**
+     * @param config   optional: when given, the constructor will instantiate the
+     *                 referenced underlying sources and check them in open()
+     */
+    VirtualSyncSource(const SyncSourceParams &params, SyncConfig *config = NULL);
+
+    /** opens underlying sources and checks config by calling getDataTypeSupport() */
+    virtual void open();
+    virtual void close();
+
+    /**
+     * returns array with source names that are referenced by this
+     * virtual source
+     */
+    std::vector<std::string> getMappedSources();
+
+    /**
+     * returns <use datatype=...> statements for XML config,
+     * throws error if not configured correctly
+     */
+    std::string getDataTypeSupport();
+    using SyncSourceBase::getDataTypeSupport;
+
+
+   /*
+    * If any of the sub datasource has no databases associated, return an empty
+    * database list to indicate a possibly error condition; otherwise return a
+    * dummy database to identify "calendar+todo" combined datasource.
+    **/
+    virtual Databases getDatabases();
+};
+
 /**
  * Hooks up the Synthesis DB Interface start sync (BeginDataRead) and
  * end sync (EndDataWrite) calls with virtual methods. Ensures that
@@ -1271,6 +1507,10 @@ class SyncSourceRevisions : virtual public SyncSourceChanges, virtual public Syn
      * should not throw errors when it cannot create a non-empty
      * string. The caller of this method will detect situations where
      * a non-empty string is necessary and none was provided.
+     *
+     * This call is typically only invoked only once during the
+     * lifetime of a source. The result returned in that invocation is
+     * used throught the session.
      */
     virtual void listAllItems(RevisionMap_t &revisions) = 0;
 
@@ -1325,18 +1565,27 @@ class SyncSourceRevisions : virtual public SyncSourceChanges, virtual public Syn
     SyncSourceDelete *m_del;
     int m_revisionAccuracySeconds;
 
+    /** buffers the result of the initial listAllItems() call */
+    RevisionMap_t m_revisions;
+    bool m_revisionsSet;
+    void initRevisions();
+
     /**
      * Dump all data from source unmodified into the given directory.
      * The ConfigNode can be used to store meta information needed for
      * restoring that state. Both directory and node are empty.
      */
-    void backupData(const string &dirname, ConfigNode &node, BackupReport &report);
+    void backupData(const SyncSource::Operations::ConstBackupInfo &oldBackup,
+                    const SyncSource::Operations::BackupInfo &newBackup,
+                    BackupReport &report);
 
     /**
      * Restore database from data stored in backupData(). Will be
      * called inside open()/close() pair. beginSync() is *not* called.
      */
-    void restoreData(const string &dirname, const ConfigNode &node, bool dryrun, SyncSourceReport &report);
+    void restoreData(const SyncSource::Operations::ConstBackupInfo &oldBackup,
+                     bool dryrun,
+                     SyncSourceReport &report);
 
     /**
      * Increments the time stamp of the latest database modification,
@@ -1406,6 +1655,88 @@ class SyncSourceLogging : public virtual SyncSourceBase
                                 const boost::function<SyncSource::Operations::DeleteItem_t> &parent);
 };
 
+/**
+ * Implements Load/SaveAdminData and MapItem handling in a SyncML
+ * server. Uses a single property for the admin data in the "internal"
+ * node and a complete node for the map items.
+ */
+class SyncSourceAdmin : public virtual SyncSourceBase
+{
+    boost::shared_ptr<ConfigNode> m_configNode;
+    std::string m_adminPropertyName;
+    boost::shared_ptr<ConfigNode> m_mappingNode;
+    bool m_mappingLoaded;
+
+    ConfigProps m_mapping;
+    ConfigProps::const_iterator m_mappingIterator;
+
+    sysync::TSyError loadAdminData(const char *aLocDB,
+                                   const char *aRemDB,
+                                   char **adminData);
+    sysync::TSyError saveAdminData(const char *adminData);
+    bool readNextMapItem(sysync::MapID mID, bool aFirst);
+    sysync::TSyError insertMapItem(sysync::cMapID mID);
+    sysync::TSyError updateMapItem(sysync::cMapID mID);
+    sysync::TSyError deleteMapItem(sysync::cMapID mID);
+    void flush();
+
+    void resetMap();
+    void mapid2entry(sysync::cMapID mID, string &key, string &value);
+    void entry2mapid(const string &key, const string &value, sysync::MapID mID);
+
+ public:
+    /** flexible initialization */
+    void init(SyncSource::Operations &ops,
+              const boost::shared_ptr<ConfigNode> &config,
+              const std::string adminPropertyName,
+              const boost::shared_ptr<ConfigNode> &mapping);
+
+    /**
+     * simpler initialization, using the default placement of data
+     * inside the SyncSourceConfig base class
+     */
+    void init(SyncSource::Operations &ops, SyncSource *source);
+};
+
+/**
+ * Implements Read/Write/DeleteBlob. Blobs are stored inside a
+ * configurable directory, which has to be unique for the current
+ * peer.
+ */
+class SyncSourceBlob : public virtual SyncSourceBase
+{
+    /**
+     * Only one blob is active at a time.
+     * This utility class provides the actual implementation.
+     */
+    sysync::TBlob m_blob;
+
+    sysync::TSyError readBlob(sysync::cItemID aID, const char *aBlobID,
+                              void **aBlkPtr, size_t *aBlkSize,
+                              size_t *aTotSize,
+                              bool aFirst, bool *aLast) {
+        return m_blob.ReadBlob(aID, aBlobID, aBlkPtr, aBlkSize, aTotSize, aFirst, aLast);
+    }
+    sysync::TSyError writeBlob(sysync::cItemID aID, const char *aBlobID,
+                               void *aBlkPtr, size_t aBlkSize,
+                               size_t aTotSize,
+                               bool aFirst, bool aLast) {
+        mkdir_p(m_blob.getBlobPath());
+        return m_blob.WriteBlob(aID, aBlobID, aBlkPtr, aBlkSize, aTotSize, aFirst, aLast);
+    }
+    sysync::TSyError deleteBlob(sysync::cItemID aID, const char *aBlobID) {
+        return m_blob.DeleteBlob(aID, aBlobID);
+    }
+
+    sysync::TSyError loadAdminData(sysync::cItemID aID, const char *aBlobID,
+                                   void **aBlkPtr, size_t *aBlkSize, size_t *aTotSize,
+                                   bool aFirst, bool *aLast);
+
+ public:
+    void init(SyncSource::Operations &ops,
+              const std::string &dir);
+};
+
 
 /**
  * This is an interface definition that is expected by the client-test
