@@ -63,7 +63,10 @@ using namespace std;
  *   (beware, such a hash might change as the textual representation
  *    changes even though the item is unchanged)
  */
-class TrackingSyncSource : public TestingSyncSource, virtual public SyncSourceRevisions
+class TrackingSyncSource : public TestingSyncSource,
+    virtual public SyncSourceRevisions,
+    virtual public SyncSourceBlob,
+    virtual public SyncSourceAdmin
 {
   public:
     /**
@@ -79,6 +82,14 @@ class TrackingSyncSource : public TestingSyncSource, virtual public SyncSourceRe
                        int granularitySeconds = 1);
     ~TrackingSyncSource() {}
 
+    /**
+     * ConfigNode used for change tracking in SyncSourceRevisions.
+     * Derived classes might need that when implementing operations
+     * which have side effects on other items (for example,
+     * EvolutionCalendarSource::removeItem()).
+     */
+    ConfigNode &getTrackingNode() { return *m_trackingNode; }
+
     /**
      * returns a list of all know sources for the kind of items
      * supported by this sync source
@@ -88,11 +99,27 @@ class TrackingSyncSource : public TestingSyncSource, virtual public SyncSourceRe
     /**
      * Actually opens the data source specified in the constructor,
      * will throw the normal exceptions if that fails. Should
-     * not modify the state of the sync source: that can be deferred
-     * until the server is also ready and beginSync() is called.
+     * not modify the state of the sync source.
+     *
+     * The expectation is that this call is fairly light-weight, but
+     * does enough checking to determine whether the source is
+     * usable. More expensive operations (like determining changes)
+     * should be done in the beginSync() callback.
+     *
+     * In clients, it will be called for all sources before
+     * the sync starts. In servers, it is called for each source once
+     * the client asks for it, but not sooner.
      */
     virtual void open() = 0;
 
+    /**
+     * A quick check whether the source currently has data. Currently
+     * used as part of the "allow slow sync" checking after open() and
+     * before beginSync(). Returning false is acceptable when it is
+     * uncertain and too expensive to check.
+     */
+    virtual bool isEmpty() = 0;
+
     /**
      * fills the complete mapping from LUID to revision string of all
      * currently existing items
@@ -193,6 +220,9 @@ class TrackingSyncSource : public TestingSyncSource, virtual public SyncSourceRe
     virtual void readItem(const std::string &luid, std::string &item);
     virtual InsertItemResult insertItemRaw(const std::string &luid, const std::string &item);
     virtual void readItemRaw(const std::string &luid, std::string &item);
+    virtual void enableServerMode();
+    virtual bool serverModeEnabled() const;
+    virtual const char *getPeerMimeType() const;
 
     boost::shared_ptr<ConfigNode> m_trackingNode;
 };
