@@ -35,6 +35,7 @@ std::string lookupDebug, lookupInfo;
 }
 
 int EDSAbiHaveEbook, EDSAbiHaveEcal, EDSAbiHaveEdataserver;
+int SyncEvoHaveLibbluetooth;
 
 #ifdef EVOLUTION_COMPATIBILITY
 
@@ -42,6 +43,12 @@ struct EDSAbiWrapper EDSAbiWrapperSingleton;
 
 namespace {
 
+enum {
+    FIND_SYMBOLS_NEED_ALL = 1<<0,              /**< all symbols must be present for findSymbols() to succeed */
+    FIND_SYMBOLS_LENIENT_MAX_VERSION = 1<<1    /**< also allow libs with higher version, with extra warning */
+};
+    
+
 /**
  * Opens a <basename>.<num> shared object with <num> coming from a
  * range of known compatible versions, falling back to even more
@@ -56,15 +63,19 @@ namespace {
  * @param libname   full name including .so suffix; .<num> gets appended
  * @param minver    first known compatible version
  * @param maxver    last known compatible version
+ * @param flags     controls specific features
+ * @retval realver  found version
  * @return dlhandle which must be kept or freed by caller
  */
-void *findSymbols(const char *libname, int minver, int maxver, ... /* function pointer address, name, ..., (void *)0 */)
+void *findSymbols(const char *libname, int minver, int maxver,
+                  int flags, int *realver, ... /* function pointer address, name, ..., (void *)0 */)
 {
     void *dlhandle = NULL;
     std::ostringstream debug, info;
+    int ver;
 
     if (!dlhandle) {
-        for (int ver = maxver;
+        for (ver = maxver;
              ver >= minver;
              --ver) {
             std::ostringstream soname;
@@ -77,8 +88,8 @@ void *findSymbols(const char *libname, int minver, int maxver, ... /* function p
         }
     }
 
-    if (!dlhandle) {
-        for (int ver = maxver + 1;
+    if (!dlhandle && (flags & FIND_SYMBOLS_LENIENT_MAX_VERSION)) {
+        for (ver = maxver + 1;
              ver < maxver + 50;
              ++ver) {
             std::ostringstream soname;
@@ -90,17 +101,21 @@ void *findSymbols(const char *libname, int minver, int maxver, ... /* function p
             }
         }
     }
-    
+
+    if (realver) {
+        *realver = ver;
+    }
+
     if (!dlhandle) {
         debug << libname << " not found (tried major versions " << minver << " to " << maxver + 49 << ")" << std::endl;
     } else {
         bool allfound = true;
 
         va_list ap;
-        va_start(ap, maxver);
+        va_start(ap, realver);
         void **funcptr = va_arg(ap, void **);
         const char *symname = NULL;
-        while (funcptr && allfound) {
+        while (funcptr) {
             symname = va_arg(ap, const char *);
             *funcptr = dlsym(dlhandle, symname);
             if (!*funcptr) {
@@ -111,9 +126,9 @@ void *findSymbols(const char *libname, int minver, int maxver, ... /* function p
         }
         va_end(ap);
 
-        if (!allfound) {
+        if (!allfound && (flags & FIND_SYMBOLS_NEED_ALL)) {
             /* unusable, clear symbols and free handle */
-            va_start(ap, maxver);
+            va_start(ap, realver);
             funcptr = va_arg(ap, void **);
             while (funcptr) {
                 va_arg(ap, const char *);
@@ -143,12 +158,18 @@ void *findSymbols(const char *libname, int minver, int maxver, ... /* function p
 # ifdef ENABLE_ECAL
     void *ecalhandle;
 # endif
+# ifdef ENABLE_BLUETOOTH
+    void *libbluetoothhandle;
+    sdp_record_t *wrap_sdp_extract_pdu(const uint8_t *pdata, int bufsize, int *scanned) { return EDSAbiWrapperSingleton.sdp_extract_pdu(pdata, scanned); }
+    int wrap_sdp_extract_seqtype(const uint8_t *buf, int bufsize, uint8_t *dtdp, int *size) { return EDSAbiWrapperSingleton.sdp_extract_seqtype(buf, dtdp, size); }
+# endif
 
 }
 
 #endif // EVOLUTION_COMPATIBILITY
 
 extern "C" int EDSAbiHaveEbook, EDSAbiHaveEcal, EDSAbiHaveEdataserver;
+extern "C" int SyncEvoHaveLibbluetooth;
 
 extern "C" void EDSAbiWrapperInit()
 {
@@ -164,6 +185,7 @@ extern "C" void EDSAbiWrapperInit()
 # ifdef HAVE_EDS
     edshandle =
     findSymbols("libedataserver-1.2.so", 7, 11,
+                FIND_SYMBOLS_NEED_ALL|FIND_SYMBOLS_LENIENT_MAX_VERSION, NULL,
                 &EDSAbiWrapperSingleton.e_source_get_type, "e_source_get_type",
                 &EDSAbiWrapperSingleton.e_source_get_uri, "e_source_get_uri",
                 &EDSAbiWrapperSingleton.e_source_group_get_type, "e_source_group_get_type",
@@ -177,6 +199,7 @@ extern "C" void EDSAbiWrapperInit()
 # ifdef ENABLE_EBOOK
     ebookhandle =
     findSymbols("libebook-1.2.so", 5, 9,
+                FIND_SYMBOLS_NEED_ALL|FIND_SYMBOLS_LENIENT_MAX_VERSION, NULL,
                 &EDSAbiWrapperSingleton.e_book_add_contact, "e_book_add_contact",
                 &EDSAbiWrapperSingleton.e_book_authenticate_user, "e_book_authenticate_user",
                 &EDSAbiWrapperSingleton.e_book_commit_contact, "e_book_commit_contact",
@@ -210,6 +233,7 @@ extern "C" void EDSAbiWrapperInit()
 # ifdef ENABLE_ECAL
     ecalhandle =
     findSymbols("libecal-1.2.so", 3, 7,
+                FIND_SYMBOLS_NEED_ALL|FIND_SYMBOLS_LENIENT_MAX_VERSION, NULL,
                 &EDSAbiWrapperSingleton.e_cal_add_timezone, "e_cal_add_timezone",
                 &EDSAbiWrapperSingleton.e_cal_component_get_icalcomponent, "e_cal_component_get_icalcomponent",
                 &EDSAbiWrapperSingleton.e_cal_component_get_last_modified, "e_cal_component_get_last_modified",
@@ -232,6 +256,7 @@ extern "C" void EDSAbiWrapperInit()
                 &EDSAbiWrapperSingleton.e_cal_remove_object_with_mod, "e_cal_remove_object_with_mod",
                 &EDSAbiWrapperSingleton.e_cal_set_auth_func, "e_cal_set_auth_func",
                 &EDSAbiWrapperSingleton.icalcomponent_add_component, "icalcomponent_add_component",
+                &EDSAbiWrapperSingleton.icalcomponent_add_property, "icalcomponent_add_property",
                 &EDSAbiWrapperSingleton.icalcomponent_as_ical_string, "icalcomponent_as_ical_string",
                 &EDSAbiWrapperSingleton.icalcomponent_free, "icalcomponent_free",
                 &EDSAbiWrapperSingleton.icalcomponent_get_first_component, "icalcomponent_get_first_component",
@@ -248,9 +273,11 @@ extern "C" void EDSAbiWrapperInit()
                 &EDSAbiWrapperSingleton.icalcomponent_new_from_string, "icalcomponent_new_from_string",
                 &EDSAbiWrapperSingleton.icalcomponent_remove_property, "icalcomponent_remove_property",
                 &EDSAbiWrapperSingleton.icalcomponent_set_uid, "icalcomponent_set_uid",
+                &EDSAbiWrapperSingleton.icalcomponent_set_recurrenceid, "icalcomponent_set_recurrenceid",
                 &EDSAbiWrapperSingleton.icalcomponent_vanew, "icalcomponent_vanew",
                 &EDSAbiWrapperSingleton.icalparameter_get_tzid, "icalparameter_get_tzid",
                 &EDSAbiWrapperSingleton.icalparameter_set_tzid, "icalparameter_set_tzid",
+                &EDSAbiWrapperSingleton.icalproperty_new_clone, "icalproperty_new_clone",
                 &EDSAbiWrapperSingleton.icalproperty_get_description, "icalproperty_get_description",
                 &EDSAbiWrapperSingleton.icalproperty_get_first_parameter, "icalproperty_get_first_parameter",
                 &EDSAbiWrapperSingleton.icalproperty_get_lastmodified, "icalproperty_get_lastmodified",
@@ -260,7 +287,10 @@ extern "C" void EDSAbiWrapperInit()
                 &EDSAbiWrapperSingleton.icalproperty_new_summary, "icalproperty_new_summary",
                 &EDSAbiWrapperSingleton.icalproperty_set_value_from_string, "icalproperty_set_value_from_string",
                 &EDSAbiWrapperSingleton.icalproperty_remove_parameter_by_kind, "icalproperty_remove_parameter_by_kind",
+                &EDSAbiWrapperSingleton.icaltime_is_null_time, "icaltime_is_null_time",
                 &EDSAbiWrapperSingleton.icaltime_as_ical_string, "icaltime_as_ical_string",
+                &EDSAbiWrapperSingleton.icaltime_from_string, "icaltime_from_string",
+                &EDSAbiWrapperSingleton.icaltime_set_timezone, "icaltime_set_timezone",
                 &EDSAbiWrapperSingleton.icaltimezone_free, "icaltimezone_free",
                 &EDSAbiWrapperSingleton.icaltimezone_get_builtin_timezone, "icaltimezone_get_builtin_timezone",
                 &EDSAbiWrapperSingleton.icaltimezone_get_builtin_timezone_from_tzid, "icaltimezone_get_builtin_timezone_from_tzid",
@@ -270,7 +300,56 @@ extern "C" void EDSAbiWrapperInit()
                 &EDSAbiWrapperSingleton.icaltimezone_set_component, "icaltimezone_set_component",
                 (void *)0);
     EDSAbiHaveEcal = EDSAbiWrapperSingleton.e_cal_new != 0;
+    ecalhandle =
+    findSymbols("libecal-1.2.so", 3, 7,
+                FIND_SYMBOLS_LENIENT_MAX_VERSION, NULL,
+                &EDSAbiWrapperSingleton.icalcomponent_as_ical_string_r, "icalcomponent_as_ical_string_r",
+                &EDSAbiWrapperSingleton.icaltime_as_ical_string_r, "icaltime_as_ical_string_r",
+                (void *)0);
 # endif // ENABLE_ECAL
+
+# ifdef ENABLE_BLUETOOTH
+    int bluetooth_version;
+    libbluetoothhandle =
+    findSymbols("libbluetooth.so", 2, 3, 0, &bluetooth_version,
+                &EDSAbiWrapperSingleton.sdp_close, "sdp_close",
+                &EDSAbiWrapperSingleton.sdp_connect, "sdp_connect",
+                &EDSAbiWrapperSingleton.sdp_extract_pdu, "sdp_extract_pdu",
+                &EDSAbiWrapperSingleton.sdp_extract_pdu_safe, "sdp_extract_pdu_safe",
+                &EDSAbiWrapperSingleton.sdp_extract_seqtype, "sdp_extract_seqtype",
+                &EDSAbiWrapperSingleton.sdp_extract_seqtype_safe, "sdp_extract_seqtype_safe",
+                &EDSAbiWrapperSingleton.sdp_get_access_protos, "sdp_get_access_protos",
+                &EDSAbiWrapperSingleton.sdp_get_proto_port, "sdp_get_proto_port",
+                &EDSAbiWrapperSingleton.sdp_get_socket, "sdp_get_socket",
+                &EDSAbiWrapperSingleton.sdp_list_append, "sdp_list_append",
+                &EDSAbiWrapperSingleton.sdp_list_free, "sdp_list_free",
+                &EDSAbiWrapperSingleton.sdp_process, "sdp_process",
+                &EDSAbiWrapperSingleton.sdp_record_free, "sdp_record_free",
+                &EDSAbiWrapperSingleton.sdp_service_search_attr_async, "sdp_service_search_attr_async",
+                &EDSAbiWrapperSingleton.sdp_set_notify, "sdp_set_notify",
+                &EDSAbiWrapperSingleton.sdp_uuid128_create, "sdp_uuid128_create",
+                &EDSAbiWrapperSingleton.str2ba, "str2ba",
+                (void *)0);
+    if (bluetooth_version == 2) {
+        // libbluetooth.so.2's sdp_extract_pdu() and
+        // sdp_extract_seqtype() do not match our prototype (have no
+        // bufsize). Some versions have a _safe variant. If not, use
+        // wrappers which accept the bufsize parameter and drop it.
+        if (!EDSAbiWrapperSingleton.sdp_extract_pdu_safe) {
+            EDSAbiWrapperSingleton.sdp_extract_pdu_safe = wrap_sdp_extract_pdu;
+        }
+        if (!EDSAbiWrapperSingleton.sdp_extract_seqtype_safe) {
+            EDSAbiWrapperSingleton.sdp_extract_seqtype_safe = wrap_sdp_extract_seqtype;
+        }
+    } else {
+        // _safe variants were removed in favor of an API/ABI change. Redirect
+        // into normal versions. We know they have the right prototype because
+        // of that change.
+        EDSAbiWrapperSingleton.sdp_extract_pdu_safe = reinterpret_cast<typeof(EDSAbiWrapperSingleton.sdp_extract_pdu_safe)>(EDSAbiWrapperSingleton.sdp_extract_pdu);
+        EDSAbiWrapperSingleton.sdp_extract_seqtype_safe = reinterpret_cast<typeof(EDSAbiWrapperSingleton.sdp_extract_seqtype_safe)>(EDSAbiWrapperSingleton.sdp_extract_seqtype);
+    }
+    SyncEvoHaveLibbluetooth = EDSAbiWrapperSingleton.sdp_connect != 0;
+# endif
 #else // EVOLUTION_COMPATIBILITY
 # ifdef HAVE_EDS
     EDSAbiHaveEdataserver = true;
@@ -281,9 +360,12 @@ extern "C" void EDSAbiWrapperInit()
 # ifdef ENABLE_ECAL
     EDSAbiHaveEcal = true;
 # endif
+# ifdef ENABLE_BLUETOOTH
+    SyncEvoHaveLibbluetooth = true;
+# endif
 #endif // EVOLUTION_COMPATIBILITY
 }
 
-extern "C" const char *EDSAbiWrapperInfo() { return lookupInfo.c_str(); }
-extern "C" const char *EDSAbiWrapperDebug() { return lookupDebug.c_str(); }
+extern "C" const char *EDSAbiWrapperInfo() { EDSAbiWrapperInit(); return lookupInfo.c_str(); }
+extern "C" const char *EDSAbiWrapperDebug() { EDSAbiWrapperInit(); return lookupDebug.c_str(); }
 
