@@ -18,7 +18,7 @@
  */
 
 /**
- * The purpose of this file is to separate SyncEvolution from
+ * The main purpose of this file is to separate SyncEvolution from
  * Evolution Dataserver ABI changes by never depending directly
  * on any symbol in its libraries. Instead all functions are
  * called via function pointers found via dlopen/dlsym.
@@ -28,9 +28,16 @@
  * may fail when the functions needed by SyncEvolution change.
  *
  * History shows that this has not happened for a long time whereas
- * the version of libs had to be bumped quite a few times due to other
+ * the versions of EDS libs had to be bumped quite a few times due to other
  * changes.
  *
+ * A similar problem came up with an API and ABI change in
+ * libbluetooth.so.3: the sdp_extract_seqtype() and sdp_extract_pdu()
+ * gained one more parameter (a buffer size for the buffer being
+ * written into). When compatibility mode is enabled, this header
+ * file provides the extended version and maps it back to the older
+ * version if necessary.
+ *
  * To use the wrappers, include this header file. It overrides
  * the normal C API functions with the function pointers via
  * defines.
@@ -56,6 +63,10 @@
 #include <libecal/e-cal.h>
 #endif
 #endif
+#ifdef ENABLE_BLUETOOTH
+#include <bluetooth/sdp.h>
+#include <bluetooth/sdp_lib.h>
+#endif
 
 #ifdef __cplusplus
 extern "C" {
@@ -64,6 +75,9 @@ extern "C" {
 /** libebook, libecal, libedataserver available (currently checks for e_book_new/e_cal_new/e_source_group_peek_sources) */
 extern int EDSAbiHaveEbook, EDSAbiHaveEcal, EDSAbiHaveEdataserver;
 
+/** libbluetooth available (checks sdp_connect()) */
+extern int SyncEvoHaveLibbluetooth;
+
 #ifdef EVOLUTION_COMPATIBILITY
 
 /**
@@ -133,6 +147,7 @@ struct EDSAbiWrapper {
     gboolean (*e_cal_remove_object_with_mod) (ECal *ecal, const char *uid, const char *rid, CalObjModType mod, GError **error);
     void (*e_cal_set_auth_func) (ECal *ecal, ECalAuthFunc func, gpointer data);
     void (*icalcomponent_add_component) (icalcomponent* parent, icalcomponent* child);
+    void (*icalcomponent_add_property) (icalcomponent* comp, icalproperty* prop);
     char* (*icalcomponent_as_ical_string) (icalcomponent* component);
     void (*icalcomponent_free) (icalcomponent* component);
     icalcomponent* (*icalcomponent_get_first_component) (icalcomponent* component, icalcomponent_kind kind);
@@ -149,9 +164,11 @@ struct EDSAbiWrapper {
     icalcomponent* (*icalcomponent_new_from_string) (char* str);
     void (*icalcomponent_remove_property) (icalcomponent* component, icalproperty* property);
     void (*icalcomponent_set_uid) (icalcomponent* comp, const char* v);
+    void (*icalcomponent_set_recurrenceid)(icalcomponent* comp, struct icaltimetype v);
     icalcomponent* (*icalcomponent_vanew) (icalcomponent_kind kind, ...);
     const char* (*icalparameter_get_tzid) (const icalparameter* value);
     void (*icalparameter_set_tzid) (icalparameter* value, const char* v);
+    icalproperty *(*icalproperty_new_clone)(icalproperty *prop);
     const char* (*icalproperty_get_description) (const icalproperty* prop);
     icalparameter* (*icalproperty_get_first_parameter) (icalproperty* prop, icalparameter_kind kind);
     struct icaltimetype (*icalproperty_get_lastmodified) (const icalproperty* prop);
@@ -163,7 +180,10 @@ struct EDSAbiWrapper {
     void (*icalproperty_set_value_from_string) (icalproperty* prop,const char* value, const char* kind);
     void (*icalproperty_remove_parameter_by_kind)(icalproperty* prop,
                                                   icalparameter_kind kind);
+    int (*icaltime_is_null_time)(const struct icaltimetype t);
     const char* (*icaltime_as_ical_string) (const struct icaltimetype tt);
+    icaltimetype (*icaltime_from_string)(const char* str);
+    void (*icaltime_set_timezone)(icaltimetype *tt, const icaltimezone *zone);
 
     void (*icaltimezone_free) (icaltimezone *zone, int free_struct);
     icaltimezone* (*icaltimezone_get_builtin_timezone) (const char *location);
@@ -172,8 +192,36 @@ struct EDSAbiWrapper {
     char* (*icaltimezone_get_tzid) (icaltimezone *zone);
     icaltimezone *(*icaltimezone_new) (void);
     int (*icaltimezone_set_component) (icaltimezone *zone, icalcomponent *comp);
+
+    // optional variants which allocate the returned string for us
+    const char* (*icaltime_as_ical_string_r) (const struct icaltimetype tt);
+    char* (*icalcomponent_as_ical_string_r) (icalcomponent* component);
 # endif /* ENABLE_ECAL */
 
+# ifdef ENABLE_BLUETOOTH
+    int (*sdp_close)(sdp_session_t *session);
+    sdp_session_t *(*sdp_connect)(const bdaddr_t *src, const bdaddr_t *dst, uint32_t flags);
+    /** libbluetooth.so.2 version of sdp_extract_pdu() */
+    sdp_record_t *(*sdp_extract_pdu)(const uint8_t *pdata, int *scanned);
+    /** alternate version of sdp_extract_pdu() only found in some releases of libbluetooth.so.2 */
+    sdp_record_t *(*sdp_extract_pdu_safe)(const uint8_t *pdata, int bufsize, int *scanned);
+    /** libbluetooth.so.2 version of sdp_extract_seqtype() */
+    int (*sdp_extract_seqtype)(const uint8_t *buf, uint8_t *dtdp, int *size);
+    /** alternate version of sdp_extract_seqtype() only found in some releases of libbluetooth.so.2 */
+    int (*sdp_extract_seqtype_safe)(const uint8_t *buf, int bufsize, uint8_t *dtdp, int *size);
+    int (*sdp_get_access_protos)(const sdp_record_t *rec, sdp_list_t **protos);
+    int (*sdp_get_proto_port)(const sdp_list_t *list, int proto);
+    int (*sdp_get_socket)(const sdp_session_t *session);
+    sdp_list_t *(*sdp_list_append)(sdp_list_t *list, void *d);
+    void        (*sdp_list_free)(sdp_list_t *list, sdp_free_func_t f);
+    int (*sdp_process)(sdp_session_t *session);
+    void (*sdp_record_free)(sdp_record_t *rec);
+    int (*sdp_service_search_attr_async)(sdp_session_t *session, const sdp_list_t *search, sdp_attrreq_type_t reqtype, const sdp_list_t *attrid_list);
+    int (*sdp_set_notify)(sdp_session_t *session, sdp_callback_t *func, void *udata);
+    uuid_t *(*sdp_uuid128_create)(uuid_t *uuid, const void *data);
+    int (*str2ba)(const char *str, bdaddr_t *ba);
+# endif /* ENABLE_BLUETOOTH */
+
     int initialized;
 };
 
@@ -241,7 +289,8 @@ extern struct EDSAbiWrapper EDSAbiWrapperSingleton;
 #   define e_cal_remove_object_with_mod EDSAbiWrapperSingleton.e_cal_remove_object_with_mod
 #   define e_cal_set_auth_func EDSAbiWrapperSingleton.e_cal_set_auth_func
 #   define icalcomponent_add_component EDSAbiWrapperSingleton.icalcomponent_add_component
-#   define icalcomponent_as_ical_string EDSAbiWrapperSingleton.icalcomponent_as_ical_string
+#   define icalcomponent_add_property EDSAbiWrapperSingleton.icalcomponent_add_property
+#   define icalcomponent_as_ical_string (EDSAbiWrapperSingleton.icalcomponent_as_ical_string_r ? EDSAbiWrapperSingleton.icalcomponent_as_ical_string_r : EDSAbiWrapperSingleton.icalcomponent_as_ical_string)
 #   define icalcomponent_free EDSAbiWrapperSingleton.icalcomponent_free
 #   define icalcomponent_get_first_component EDSAbiWrapperSingleton.icalcomponent_get_first_component
 #   define icalcomponent_get_first_property EDSAbiWrapperSingleton.icalcomponent_get_first_property
@@ -257,9 +306,11 @@ extern struct EDSAbiWrapper EDSAbiWrapperSingleton;
 #   define icalcomponent_new_from_string EDSAbiWrapperSingleton.icalcomponent_new_from_string
 #   define icalcomponent_remove_property EDSAbiWrapperSingleton.icalcomponent_remove_property
 #   define icalcomponent_set_uid EDSAbiWrapperSingleton.icalcomponent_set_uid
+#   define icalcomponent_set_recurrenceid EDSAbiWrapperSingleton.icalcomponent_set_recurrenceid
 #   define icalcomponent_vanew EDSAbiWrapperSingleton.icalcomponent_vanew
 #   define icalparameter_get_tzid EDSAbiWrapperSingleton.icalparameter_get_tzid
 #   define icalparameter_set_tzid EDSAbiWrapperSingleton.icalparameter_set_tzid
+#   define icalproperty_new_clone EDSAbiWrapperSingleton.icalproperty_new_clone
 #   define icalproperty_get_description EDSAbiWrapperSingleton.icalproperty_get_description
 #   define icalproperty_get_first_parameter EDSAbiWrapperSingleton.icalproperty_get_first_parameter
 #   define icalproperty_get_lastmodified EDSAbiWrapperSingleton.icalproperty_get_lastmodified
@@ -269,7 +320,10 @@ extern struct EDSAbiWrapper EDSAbiWrapperSingleton;
 #   define icalproperty_new_summary EDSAbiWrapperSingleton.icalproperty_new_summary
 #   define icalproperty_set_value_from_string EDSAbiWrapperSingleton.icalproperty_set_value_from_string
 #   define icalproperty_remove_parameter_by_kind EDSAbiWrapperSingleton.icalproperty_remove_parameter_by_kind
-#   define icaltime_as_ical_string EDSAbiWrapperSingleton.icaltime_as_ical_string
+#   define icaltime_is_null_time EDSAbiWrapperSingleton.icaltime_is_null_time
+#   define icaltime_as_ical_string (EDSAbiWrapperSingleton.icaltime_as_ical_string_r ? EDSAbiWrapperSingleton.icaltime_as_ical_string_r : EDSAbiWrapperSingleton.icaltime_as_ical_string)
+#   define icaltime_from_string EDSAbiWrapperSingleton.icaltime_from_string
+#   define icaltime_set_timezone EDSAbiWrapperSingleton.icaltime_set_timezone
 #   define icaltimezone_free EDSAbiWrapperSingleton.icaltimezone_free
 #   define icaltimezone_get_builtin_timezone EDSAbiWrapperSingleton.icaltimezone_get_builtin_timezone
 #   define icaltimezone_get_builtin_timezone_from_tzid EDSAbiWrapperSingleton.icaltimezone_get_builtin_timezone_from_tzid
@@ -278,8 +332,39 @@ extern struct EDSAbiWrapper EDSAbiWrapperSingleton;
 #   define icaltimezone_new EDSAbiWrapperSingleton.icaltimezone_new
 #   define icaltimezone_set_component EDSAbiWrapperSingleton.icaltimezone_set_component
 #  endif /* ENABLE_ECAL */
+#  ifdef ENABLE_BLUETOOTH
+#   define sdp_close EDSAbiWrapperSingleton.sdp_close
+#   define sdp_connect EDSAbiWrapperSingleton.sdp_connect
+#   define sdp_extract_pdu do_not_use_sdp_extract_pdu
+#   define sdp_extract_pdu_safe EDSAbiWrapperSingleton.sdp_extract_pdu_safe
+#   define sdp_extract_seqtype do_not_use_sdp_extract_seqtype
+#   define sdp_extract_seqtype_safe EDSAbiWrapperSingleton.sdp_extract_seqtype_safe
+#   define sdp_get_access_protos EDSAbiWrapperSingleton.sdp_get_access_protos
+#   define sdp_get_proto_port EDSAbiWrapperSingleton.sdp_get_proto_port
+#   define sdp_get_socket EDSAbiWrapperSingleton.sdp_get_socket
+#   define sdp_list_append EDSAbiWrapperSingleton.sdp_list_append
+#   define sdp_list_free EDSAbiWrapperSingleton.sdp_list_free
+#   define sdp_process EDSAbiWrapperSingleton.sdp_process
+#   define sdp_record_free EDSAbiWrapperSingleton.sdp_record_free
+#   define sdp_service_search_attr_async EDSAbiWrapperSingleton.sdp_service_search_attr_async
+#   define sdp_set_notify EDSAbiWrapperSingleton.sdp_set_notify
+#   define sdp_uuid128_create EDSAbiWrapperSingleton.sdp_uuid128_create
+#   define str2ba EDSAbiWrapperSingleton.str2ba
+#  endif /* ENABLE_BLUETOOTH */
 # endif /* EDS_ABI_WRAPPER_NO_REDEFINE */
 
+#else /* EVOLUTION_COMPATIBILITY */
+
+# if !defined(EDS_ABI_WRAPPER_NO_REDEFINE) && defined(HAVE_LIBICAL_R)
+#  ifdef ENABLE_ECAL
+#   ifndef LIBICAL_MEMFIXES
+     /* This changes the semantic of the normal functions the same way as libecal did. */
+#    define LIBICAL_MEMFIXES 1
+#    define icaltime_as_ical_string icaltime_as_ical_string_r
+#    define icalcomponent_as_ical_string icalcomponent_as_ical_string_r
+#   endif /* LIBICAL_MEMFIXES */
+#  endif /* ENABLE_ECAL */
+# endif /* EDS_ABI_WRAPPER_NO_REDEFINE */
 #endif /* EVOLUTION_COMPATIBILITY */
 
 /** initialize pointers to EDS functions, if necessary; can be called multiple times */
