@@ -70,7 +70,8 @@ class CheckSyncReport {
         serverUpdated(srUpdated),
         serverDeleted(srDeleted),
         mustSucceed(mstSucceed),
-        syncMode(mode)
+        syncMode(mode),
+        m_report(NULL)
         {}
 
     virtual ~CheckSyncReport() {}
@@ -80,6 +81,13 @@ class CheckSyncReport {
     bool mustSucceed;
     SyncMode syncMode;
 
+    // if set, then the report is copied here
+    SyncReport *m_report;
+
+    CheckSyncReport &setMode(SyncMode mode) { syncMode = mode; return *this; }
+    CheckSyncReport &setReport(SyncReport *report) { m_report = report; return *this; }
+
+
     /**
      * checks that the sync completed as expected and throws
      * CPPUnit exceptions if something is wrong
@@ -94,6 +102,10 @@ class CheckSyncReport {
  * parameters for running a sync
  */
 struct SyncOptions {
+    /** default maximum message size */
+    static const long DEFAULT_MAX_MSG_SIZE = 128 * 1024;
+    /** default maximum object size */
+    static const long DEFAULT_MAX_OBJ_SIZE = 1024 * 1024 * 1024;
     /** sync mode chosen by client */
     SyncMode m_syncMode;
     /**
@@ -111,6 +123,9 @@ struct SyncOptions {
     bool m_loSupport;
     /** enabled WBXML (default) */
     bool m_isWBXML;
+    /** overrides resend properties */
+    int m_retryDuration;
+    int m_retryInterval;
 
     bool m_isSuspended; 
     
@@ -129,8 +144,8 @@ struct SyncOptions {
 
     SyncOptions(SyncMode syncMode = SYNC_NONE,
                 const CheckSyncReport &checkReport = CheckSyncReport(),
-                long maxMsgSize = 128 * 1024, // 128KB = large enough that normal tests should run with a minimal number of messages
-                long maxObjSize = 1 * 1024 * 1024 * 1024, // 1GB = basically unlimited...
+                long maxMsgSize = DEFAULT_MAX_MSG_SIZE, // 128KB = large enough that normal tests should run with a minimal number of messages
+                long maxObjSize = DEFAULT_MAX_OBJ_SIZE, // 1GB = basically unlimited...
                 bool loSupport = false,
                 bool isWBXML = defaultWBXML(),
                 Callback_t startCallback = EmptyCallback,
@@ -142,6 +157,8 @@ struct SyncOptions {
         m_maxObjSize(maxObjSize),
         m_loSupport(loSupport),
         m_isWBXML(isWBXML),
+        m_retryDuration(300),
+        m_retryInterval(60),
         m_isSuspended(false),
         m_isAborted(false),
         m_startCallback(startCallback),
@@ -154,6 +171,8 @@ struct SyncOptions {
     SyncOptions &setMaxObjSize(long maxObjSize) { m_maxObjSize = maxObjSize; return *this; }
     SyncOptions &setLOSupport(bool loSupport) { m_loSupport = loSupport; return *this; }
     SyncOptions &setWBXML(bool isWBXML) { m_isWBXML = isWBXML; return *this; }
+    SyncOptions &setRetryDuration(int retryDuration) { m_retryDuration = retryDuration; return *this; }
+    SyncOptions &setRetryInterval(int retryInterval) { m_retryInterval = retryInterval; return *this; }
     SyncOptions &setStartCallback(const Callback_t &callback) { m_startCallback = callback; return *this; }
     SyncOptions &setTransportAgent(const boost::shared_ptr<TransportAgent> transport)
                                   {m_transport = transport; return *this;}
@@ -287,15 +306,30 @@ class ClientTest {
      * Data sources are enumbered from 0 to n-1 for the purpose of
      * testing. This call returns n.
      */
-    virtual int getNumSources() = 0;
+    virtual int getNumLocalSources() = 0;
+    virtual int getNumSyncSources() = 0;
 
     /**
      * Called to fill the given test source config with information
      * about a sync source identified by its index. It's okay to only
      * fill in the available pieces of information and set everything
      * else to zero.
+     * Two kinds of source config indexs are maintained, used for localSources
+     * and SyncSources, this is because virtual datasoures should be visible as
+     * a whole to the synccontext while should be viewed as a list of sub
+     * datasoures for Localtests.
      */
-    virtual void getSourceConfig(int source, Config &config) = 0;
+    virtual void getLocalSourceConfig(int source, Config &config) = 0;
+    virtual void getSyncSourceConfig(int source, Config &config) = 0;
+
+    /**
+     * Find the correspoding test source config via config name.
+     */
+    virtual void getSourceConfig(const string &configName, Config &config) =0;
+    /*
+     * Give me a test source config name, return the index in localSyncSources.
+     * */
+    virtual int getLocalSourcePosition (const string &configName) =0;
 
     /**
      * The instance to use as second client. Returning NULL disables
@@ -661,13 +695,25 @@ protected:
 
     virtual void doInterruptResume(int changes,
                   boost::shared_ptr<TransportWrapper> wrapper); 
+
+    /**
+     * CLIENT_ = change made on client B before interrupting
+     * SERVER_ = change made on client A and applied to server before interrupting
+     *           while sending to B
+     * _ADD = new item added
+     * _REMOVE = existing item deleted
+     * _UPDATE = existing item replaced
+     * BIG = when adding or updating, make the new item so large that it does
+     *       not fit into a single message
+     */
     enum {
         CLIENT_ADD = (1<<0),
         CLIENT_REMOVE = (1<<1),
         CLIENT_UPDATE = (1<<2),
         SERVER_ADD = (1<<3),
         SERVER_REMOVE = (1<<4),
-        SERVER_UPDATE = (1<<5)
+        SERVER_UPDATE = (1<<5),
+        BIG = (1<<6)
     };
     virtual void testInterruptResumeClientAdd();
     virtual void testInterruptResumeClientRemove();
@@ -675,6 +721,10 @@ protected:
     virtual void testInterruptResumeServerAdd();
     virtual void testInterruptResumeServerRemove();
     virtual void testInterruptResumeServerUpdate();
+    virtual void testInterruptResumeClientAddBig();
+    virtual void testInterruptResumeClientUpdateBig();
+    virtual void testInterruptResumeServerAddBig();
+    virtual void testInterruptResumeServerUpdateBig();
     virtual void testInterruptResumeFull();
 
     virtual void testUserSuspendClientAdd();
@@ -683,6 +733,10 @@ protected:
     virtual void testUserSuspendServerAdd();
     virtual void testUserSuspendServerRemove();
     virtual void testUserSuspendServerUpdate();
+    virtual void testUserSuspendClientAddBig();
+    virtual void testUserSuspendClientUpdateBig();
+    virtual void testUserSuspendServerAddBig();
+    virtual void testUserSuspendServerUpdateBig();
     virtual void testUserSuspendFull();
 
     virtual void testResendClientAdd();
@@ -739,25 +793,21 @@ public:
     virtual int getMessageCount() { return m_messageCount; }
 
     virtual void setURL(const std::string &url) { m_wrappedAgent->setURL(url); }
-    virtual void setProxy(const std::string &proxy) { m_wrappedAgent->setProxy(proxy); }
-    virtual void setProxyAuth(const std::string &user,
-                              const std::string &password) { m_wrappedAgent->setProxyAuth(user, password); }
-    virtual void setSSL(const std::string &cacerts,
-                        bool verifyServer,
-                        bool verifyHost) { m_wrappedAgent->setSSL(cacerts, verifyServer, verifyHost); }
     virtual void setContentType(const std::string &type) { m_wrappedAgent->setContentType(type); }
-    virtual void setUserAgent(const::string &agent) { m_wrappedAgent->setUserAgent(agent); }
     virtual void setAgent(boost::shared_ptr<TransportAgent> agent) {m_wrappedAgent = agent;}
     virtual void setSyncOptions(SyncOptions *options) {m_options = options;}
     virtual void setInterruptAtMessage (int interrupt) {m_interruptAtMessage = interrupt;}
     virtual void cancel() { m_wrappedAgent->cancel(); }
-    virtual void reset() {
+    virtual void shutdown() { m_wrappedAgent->shutdown(); }
+
+    virtual void rewind() {
         m_messageCount = 0;
         m_interruptAtMessage = -1;
         m_status = INACTIVE;
         m_options = NULL;
+        m_wrappedAgent.reset();
     }
-    virtual Status wait() { return m_status; }
+    virtual Status wait(bool noReply = false) { return m_status; }
     virtual void setCallback (TransportCallback cb, void *udata, int interval) 
     { return m_wrappedAgent->setCallback(cb, udata, interval);}
 };
