@@ -179,7 +179,7 @@ class Context:
         sys.stdout.flush()
         result = os.system(cmd)
         if result != 0:
-            raise Exception("%s: failed (return code %d)" % (cmd, result))
+            raise Exception("%s: failed (return code %d)" % (cmd, result>>8))
 
     def add(self, action):
         """Add an action for later execution. Order is important, fifo..."""
@@ -259,21 +259,24 @@ class Context:
         # run testresult checker 
         #calculate the src dir where client-test can be located
         srcdir = os.path.join(self.tmpdir,"build/src")
-        backendir = os.path.join(self.tmpdir, "install/usr/lib/syncevolution/backends")
-        os.system ("resultchecker.py " +self.resultdir+" "+",".join(run_servers)+" "+self.uri +" "+srcdir + " '" + options.shell + " " + options.testprefix +" '"+" '" +backendir +"'");
+        backenddir = os.path.join(self.tmpdir, "install/usr/lib/syncevolution/backends")
+        os.system ("resultchecker.py " +self.resultdir+" "+",".join(run_servers)+" "+self.uri +" "+srcdir + " '" + options.shell + " " + options.testprefix +" '"+" '" +backenddir +"'");
         # transform to html
         os.system ("xsltproc -o " + self.resultdir + "/cmp_result.xml --stringparam cmp_file " + self.lastresultdir +"/nightly.xml "+self.datadir +"/compare.xsl "+ self.resultdir+"/nightly.xml")
         os.system ("xsltproc -o " + self.resultdir + "/nightly.html --stringparam cmp_result_file " + self.resultdir + "/cmp_result.xml " + self.datadir +"/generate-html.xsl "+ self.resultdir+"/nightly.xml")
         # report result by email
         if self.recipients:
             server = smtplib.SMTP(self.mailhost)
-            resulthtml = open (self.resultdir + "/nightly.html")
-            line=resulthtml.readline()
             msg=''
-            while(line!=''):
+            try:
+                resulthtml = open (self.resultdir + "/nightly.html")
+                line=resulthtml.readline()
+                while(line!=''):
                     msg=msg+line
                     line=resulthtml.readline()
-            resulthtml.close()
+                resulthtml.close()
+            except IOError:
+                msg = '''<html><body><h1>Error: No HTML report generated!</h1></body></html>\n'''
             body = StringIO.StringIO()
             writer = MimeWriter.MimeWriter (body)
             writer.addheader("From", self.sender)
@@ -369,10 +372,15 @@ class GitCheckout(Action):
 
     def execute(self):
         if os.access(self.basedir, os.F_OK):
-            cmd = "cd %s && git fetch %s && git checkout %s && git pull" % (self.basedir, self.url, self.revision)
+            cmd = "cd %s && git fetch" % (self.basedir)
         else:
-            cmd = "git clone %s %s && cd %s && git checkout %s" % (self.url, self.basedir, self.basedir, self.revision)
+            cmd = "git clone %s %s" % (self.url, self.basedir)
         context.runCommand(cmd)
+        context.runCommand("set -x; cd %(dir)s && git show-ref &&"
+                           "((git tag -l | grep -w -q %(rev)s) && git checkout %(rev)s ||"
+                           "((git branch -l | grep -w -q %(rev)s) && git checkout %(rev)s || git checkout -b %(rev)s origin/%(rev)s) && git merge origin/%(rev)s)" %
+                           {"dir": self.basedir,
+                            "rev": self.revision})
         os.chdir(self.basedir)
         if os.access("autogen.sh", os.F_OK):
             context.runCommand("%s ./autogen.sh" % (self.runner))
@@ -420,12 +428,17 @@ class SyncEvolutionTest(Action):
     def execute(self):
         resdir = os.getcwd()
         os.chdir(self.srcdir)
+        # clear previous test results
+        context.runCommand("%s testclean" % context.make)
         try:
             if context.setupcmd:
-                context.runCommand("%s %s %s %s ./syncevolution" % (self.testenv, self.runner, context.setupcmd, self.name))
-            backendir = os.path.join(context.tmpdir, "install/usr/lib/syncevolution/backends")
-            basecmd = "CLIENT_TEST_SERVER=%s CLIENT_TEST_SOURCES=%s %s SYNCEVOLUTION_BACKEND_DIR=%s SYNC_EVOLUTION_EVO_CALENDAR_DELAY=1 CLIENT_TEST_ALARM=1200 CLIENT_TEST_LOG=%s CLIENT_TEST_EVOLUTION_PREFIX=file://%s/databases %s %s env LD_LIBRARY_PATH=build-synthesis/src/.libs ./client-test" % (self.serverName, ",".join(self.sources), self.testenv, backendir, self.serverlogs, context.workdir, self.runner, self.testPrefix);
-            context.runCommand("%s testclean" % context.make)
+                cmd = "%s %s %s %s ./syncevolution" % (self.testenv, self.runner, context.setupcmd, self.name)
+                context.runCommand("%s || sleep 5 && %s" % (cmd, cmd))
+            backenddir = os.path.join(context.tmpdir, "install/usr/lib/syncevolution/backends")
+            if not os.access(backenddir, os.F_OK):
+                # try relative to client-test inside the current directory
+                backenddir = "backends"
+            basecmd = "CLIENT_TEST_SERVER=%s CLIENT_TEST_SOURCES=%s %s SYNCEVOLUTION_BACKEND_DIR=%s SYNC_EVOLUTION_EVO_CALENDAR_DELAY=1 CLIENT_TEST_ALARM=1200 CLIENT_TEST_LOG=%s CLIENT_TEST_EVOLUTION_PREFIX=file://%s/databases %s env LD_LIBRARY_PATH=build-synthesis/src/.libs %s ./client-test" % (self.serverName, ",".join(self.sources), self.testenv, backenddir, self.serverlogs, context.workdir, self.runner, self.testPrefix);
             if self.tests:
                 tests = []
                 for test in self.tests:
@@ -605,7 +618,10 @@ class SyncEvolutionCheckout(GitCheckout):
     def __init__(self, name, revision):
         """checkout SyncEvolution"""
         GitCheckout.__init__(self,
-                             name, context.workdir, options.shell,
+                             name, context.workdir,
+                             # parameter to autogen.sh in SyncEvolution: also
+                             # check for clean Synthesis source
+                             "env SYNTHESISSRC=../libsynthesis %s" % options.shell,
                              "git@git.moblin.org:syncevolution.git",
                              revision)
 
@@ -860,7 +876,10 @@ class FunambolTest(SyncEvolutionTest):
                                    "CLIENT_TEST_XML=1 "
                                    "CLIENT_TEST_MAX_ITEMSIZE=2048 "
                                    "CLIENT_TEST_DELAY=10 "
-                                   "CLIENT_TEST_FAILURES= "
+                                   "CLIENT_TEST_FAILURES="
+                                   "Client::Sync::vcard21::testTwinning,"
+                                   "Client::Sync::vcard21_ical20_itodo20_text::testTwinning,"
+                                   "Client::Sync::text_itodo20_ical20_vcard21::testTwinning "
                                    "CLIENT_TEST_COMPARE_LOG=T "
                                    "CLIENT_TEST_RESEND_TIMEOUT=5 "
                                    "CLIENT_TEST_INTERRUPT_AT=1",
@@ -940,12 +959,14 @@ mobicaltest = SyncEvolutionTest("mobical", compile,
                                 "Client::Sync::vcard21::testDelete,"
                                 "Client::Sync::vcard21::testItemsXML,"
                                 "Client::Sync::vcard21::testOneWayFromServer,"
+                                "Client::Sync::vcard21::testOneWayFromClient,"
                                 "Client::Sync::ical20::testRefreshFromClientSync,"
                                 "Client::Sync::ical20::testSlowSyncSemantic,"
                                 "Client::Sync::ical20::testRefreshStatus,"
                                 "Client::Sync::ical20::testDelete,"
                                 "Client::Sync::ical20::testItemsXML,"
                                 "Client::Sync::ical20::testOneWayFromServer,"
+                                "Client::Sync::ical20::testOneWayFromClient,"
                                 "Client::Sync::ical20::Retry,"
                                 "Client::Sync::ical20::Suspend,"
                                 "Client::Sync::ical20::Resend,"
@@ -955,6 +976,7 @@ mobicaltest = SyncEvolutionTest("mobical", compile,
                                 "Client::Sync::itodo20::testDelete,"
                                 "Client::Sync::itodo20::testItemsXML,"
                                 "Client::Sync::itodo20::testOneWayFromServer,"
+                                "Client::Sync::itodo20::testOneWayFromClient,"
                                 "Client::Sync::itodo20::Retry,"
                                 "Client::Sync::itodo20::Suspend,"
                                 "Client::Sync::itodo20::Resend,"
@@ -964,6 +986,7 @@ mobicaltest = SyncEvolutionTest("mobical", compile,
                                 "Client::Sync::text::testDelete,"
                                 "Client::Sync::text::testItemsXML,"
                                 "Client::Sync::text::testOneWayFromServer,"
+                                "Client::Sync::text::testOneWayFromClient,"
                                 "Client::Sync::text::Retry,"
                                 "Client::Sync::text::Suspend,"
                                 "Client::Sync::text::Resend,"
@@ -973,6 +996,7 @@ mobicaltest = SyncEvolutionTest("mobical", compile,
                                 "Client::Sync::vcard21_ical20_itodo20_text::testDelete,"
                                 "Client::Sync::vcard21_ical20_itodo20_text::testItemsXML,"
                                 "Client::Sync::vcard21_ical20_itodo20_text::testOneWayFromServer,"
+                                "Client::Sync::vcard21_ical20_itodo20_text::testOneWayFromClient,"
                                 "Client::Sync::vcard21_ical20_itodo20_text::Retry,"
                                 "Client::Sync::vcard21_ical20_itodo20_text::Suspend,"
                                 "Client::Sync::vcard21_ical20_itodo20_text::Resend,"
@@ -982,6 +1006,7 @@ mobicaltest = SyncEvolutionTest("mobical", compile,
                                 "Client::Sync::text_itodo20_ical20_vcard21::testDelete,"
                                 "Client::Sync::text_itodo20_ical20_vcard21::testItemsXML,"
                                 "Client::Sync::text_itodo20_ical20_vcard21::testOneWayFromServer,"
+                                "Client::Sync::text_itodo20_ical20_vcard21::testOneWayFromClient,"
                                 "Client::Sync::text_itodo20_ical20_vcard21::Retry,"
                                 "Client::Sync::text_itodo20_ical20_vcard21::Suspend,"
                                 "Client::Sync::text_itodo20_ical20_vcard21::Resend "
@@ -1007,6 +1032,7 @@ memotootest = SyncEvolutionTest("memotoo", compile,
                                 "Client::Sync::vcard21::testRefreshFromClientSync,"
                                 "Client::Sync::vcard21::testRefreshFromClientSemantic,"
                                 "Client::Sync::vcard21::testRefreshStatus,"
+                                "Client::Sync::vcard21::testDeleteAllRefresh,"
                                 "Client::Sync::vcard21::testOneWayFromServer,"
                                 "Client::Sync::ical20::testRefreshFromClientSync,"
                                 "Client::Sync::ical20::testRefreshFromClientSemantic,"
@@ -1017,18 +1043,21 @@ memotootest = SyncEvolutionTest("memotoo", compile,
                                 "Client::Sync::itodo20::testRefreshFromClientSync,"
                                 "Client::Sync::itodo20::testRefreshFromClientSemantic,"
                                 "Client::Sync::itodo20::testRefreshStatus,"
+                                "Client::Sync::itodo20::testDeleteAllRefresh,"
                                 "Client::Sync::itodo20::testOneWayFromServer,"
                                 "Client::Sync::itodo20::Retry,"
                                 "Client::Sync::itodo20::Suspend,"
                                 "Client::Sync::text::testRefreshFromClientSync,"
                                 "Client::Sync::text::testRefreshFromClientSemantic,"
                                 "Client::Sync::text::testRefreshStatus,"
+                                "Client::Sync::text::testDeleteAllRefresh,"
                                 "Client::Sync::text::testOneWayFromServer,"
                                 "Client::Sync::text::Retry,"
                                 "Client::Sync::text::Suspend,"
                                 "Client::Sync::vcard21_ical20_itodo20_text::testRefreshFromClientSync,"
                                 "Client::Sync::vcard21_ical20_itodo20_text::testRefreshFromClientSemantic,"
                                 "Client::Sync::vcard21_ical20_itodo20_text::testRefreshStatus,"
+                                "Client::Sync::vcard21_ical20_itodo20_text::testDeleteAllRefresh,"
                                 "Client::Sync::vcard21_ical20_itodo20_text::testOneWayFromServer,"
                                 "Client::Sync::vcard21_ical20_itodo20_text::Retry,"
                                 "Client::Sync::vcard21_ical20_itodo20_text::Suspend,"
@@ -1036,6 +1065,7 @@ memotootest = SyncEvolutionTest("memotoo", compile,
                                 "Client::Sync::text_itodo20_ical20_vcard21::testRefreshFromClientSemantic,"
                                 "Client::Sync::text_itodo20_ical20_vcard21::testRefreshStatus,"
                                 "Client::Sync::text_itodo20_ical20_vcard21::testOneWayFromServer,"
+                                "Client::Sync::text_itodo20_ical20_vcard21::testDeleteAllRefresh,"
                                 "Client::Sync::text_itodo20_ical20_vcard21::Retry,"
                                 "Client::Sync::text_itodo20_ical20_vcard21::Suspend "
                                 "CLIENT_TEST_DELAY=5 "
@@ -1045,6 +1075,63 @@ memotootest = SyncEvolutionTest("memotoo", compile,
                                 testPrefix=options.testprefix)
 context.add(memotootest)
 
+ovitest = SyncEvolutionTest("ovi", compile,
+                                "", options.shell,
+                                [ "Client::Sync" ],
+                                [ "vcard30",
+                                  "calendar+todo" ],
+                                "CLIENT_TEST_DELETE_REFRESH=1 "
+                                "CLIENT_TEST_NUM_ITEMS=50 "
+                                "CLIENT_TEST_MAX_ITEMSIZE=512 "
+                                "CLIENT_TEST_SKIP="
+                                "Client::Sync::vcard30::Retry,"
+                                "Client::Sync::vcard30::Suspend,"
+                                "Client::Sync::vcard30::testOneWayFromClient,"
+                                "Client::Sync::vcard30::testOneWayFromServer,"
+                                "Client::Sync::vcard30::testSlowSyncSemantic,"
+                                "Client::Sync::vcard30::testComplexRefreshFromServerSemantic,"
+                                "Client::Sync::vcard30::testDelete,"
+                                "Client::Sync::vcard30::testDeleteAllSync,"
+                                "Client::Sync::vcard30::testManyDeletes,"
+                                "Client::Sync::calendar+todo::Retry,"
+                                "Client::Sync::calendar+todo::Suspend,"
+                                "Client::Sync::calendar+todo::testOneWayFromClient,"
+                                "Client::Sync::calendar+todo::testOneWayFromServer,"
+                                "Client::Sync::calendar+todo::testSlowSyncSemantic,"
+                                "Client::Sync::calendar+todo::testComplexRefreshFromServerSemantic,"
+                                "Client::Sync::calendar+todo::testDelete,"
+                                "Client::Sync::calendar+todo::testDeleteAllSync,"
+                                "Client::Sync::calendar+todo::testManyDeletes,"
+                                "Client::Sync::calendar+todo::testDeleteAllRefresh,"
+                                "Client::Sync::calendar+todo::testItemsXML,"
+                                "Client::Sync::calendar+todo::testMaxMsg,"
+                                "Client::Sync::calendar+todo::testLargeObject,"
+                                "Client::Sync::calendar+todo_vcard30::Retry,"
+                                "Client::Sync::calendar+todo_vcard30::Suspend,"
+                                "Client::Sync::calendar+todo_vcard30::testOneWayFromClient,"
+                                "Client::Sync::calendar+todo_vcard30::testOneWayFromServer,"
+                                "Client::Sync::calendar+todo_vcard30::testSlowSyncSemantic,"
+                                "Client::Sync::calendar+todo_vcard30::testComplexRefreshFromServerSemantic,"
+                                "Client::Sync::calendar+todo_vcard30::testDelete,"
+                                "Client::Sync::calendar+todo_vcard30::testDeleteAllSync,"
+                                "Client::Sync::calendar+todo_vcard30::testManyDeletes,"
+                                "Client::Sync::calendar+todo::Retry,"
+                                "Client::Sync::vcard30_calendar+todo::Suspend,"
+                                "Client::Sync::vcard30_calendar+todo::testOneWayFromClient,"
+                                "Client::Sync::vcard30_calendar+todo::testOneWayFromServer,"
+                                "Client::Sync::vcard30_calendar+todo::testSlowSyncSemantic,"
+                                "Client::Sync::vcard30_calendar+todo::testComplexRefreshFromServerSemantic,"
+                                "Client::Sync::vcard30_calendar+todo::testDelete,"
+                                "Client::Sync::vcard30_calendar+todo::testDeleteAllSync,"
+                                "Client::Sync::vcard30_calendar+todo::testManyDeletes,"
+                                "CLIENT_TEST_DELAY=5 "
+                                "CLIENT_TEST_COMPARE_LOG=T "
+                                "CLIENT_TEST_RESEND_TIMEOUT=5 "
+                                "CLIENT_TEST_INTERRUPT_AT=1",
+                                serverName="Ovi",
+                                testPrefix=options.testprefix)
+context.add(ovitest)
+
 if options.list:
     for action in context.todo:
         print action.name
